@@ -8,14 +8,16 @@
  ***************************************************************************/
 
 // qt/kde includes
-#include <qdom.h>
-#include <qcolor.h>
-#include <qapplication.h>
+#include <QtGui/QApplication>
+#include <QtGui/QColor>
+
 #include <kdebug.h>
+#include <klocale.h>
 
 // local includes
 #include "annotations.h"
 
+using namespace Okular;
 
 //BEGIN AnnotationUtils implementation
 Annotation * AnnotationUtils::createAnnotation( const QDomElement & annElement )
@@ -78,6 +80,41 @@ QDomElement AnnotationUtils::findChildElement( const QDomNode & parentNode,
     // if the name can't be found, return a dummy null element
     return QDomElement();
 }
+
+QString AnnotationUtils::captionForAnnotation( Annotation * ann )
+{
+    if ( !ann )
+        return QString();
+
+    QString ret;
+    switch( ann->subType() )
+    {
+        case Okular::Annotation::AText:
+            if( ( (Okular::TextAnnotation*)ann )->textType == Okular::TextAnnotation::Linked )
+                ret = i18n( "Note" );
+            else
+                ret = i18n( "FreeText" );
+            break;
+        case Okular::Annotation::ALine:
+            ret = i18n( "Line" );
+            break;
+        case Okular::Annotation::AGeom:
+            ret = i18n( "Geom" );
+            break;
+        case Okular::Annotation::AHighlight:
+            ret = i18n( "Highlight" );
+            break;
+        case Okular::Annotation::AStamp:
+            ret = i18n( "Stamp" );
+            break;
+        case Okular::Annotation::AInk:
+            ret = i18n( "Ink" );
+            break;
+        case Okular::Annotation::A_BASE:
+            break;
+    }
+    return ret;
+}
 //END AnnotationUtils implementation
 
 
@@ -310,14 +347,16 @@ void Annotation::store( QDomNode & annNode, QDomDocument & document ) const
 
 
 /** TextAnnotation [Annotation] */
-
+/*
+  The default textIcon for text annotation is Note as the PDF Reference says
+*/
 TextAnnotation::TextAnnotation()
-    : Annotation(), textType( Linked ), textIcon( "Comment" ),
+    : Annotation(), textType( Linked ), textIcon( "Note" ),
     inplaceAlign( 0 ), inplaceIntent( Unknown )
 {}
 
 TextAnnotation::TextAnnotation( const QDomNode & node )
-    : Annotation( node ), textType( Linked ), textIcon( "Comment" ),
+    : Annotation( node ), textType( Linked ), textIcon( "Note" ),
     inplaceAlign( 0 ), inplaceIntent( Unknown )
 {
     // loop through the whole children looking for a 'text' element
@@ -660,11 +699,11 @@ void HighlightAnnotation::store( QDomNode & node, QDomDocument & document ) cons
 /** StampAnnotation [Annotation] */
 
 StampAnnotation::StampAnnotation()
-    : Annotation(), stampIconName( "okular" )
+    : Annotation(), stampIconName( "Draft" )
 {}
 
 StampAnnotation::StampAnnotation( const QDomNode & node )
-    : Annotation( node ), stampIconName( "okular" )
+    : Annotation( node ), stampIconName( "Draft" )
 {
     // loop through the whole children looking for a 'stamp' element
     QDomNode subNode = node.firstChild();
@@ -694,7 +733,7 @@ void StampAnnotation::store( QDomNode & node, QDomDocument & document ) const
     node.appendChild( stampElement );
 
     // append the optional attributes
-    if ( stampIconName != "okular" )
+    if ( stampIconName != "Draft" )
         stampElement.setAttribute( "icon", stampIconName );
 }
 
