@@ -12,19 +12,36 @@
 
 #include "okular_export.h"
 
-#include <qrect.h>
+class QRect;
+
+namespace Okular
+{
 
 /**
- * Namespace with utility functions
+ * @short General utility functions.
+ *
+ * This class contains some static functions of general utility.
  */
-namespace okularUtils
+class OKULAR_EXPORT Utils
 {
+  public:
     /**
      * Rotate the rect \p source in the area \p width x \p height with the
      * specified \p orientation .
      */
-    QRect OKULAR_EXPORT rotateRect( const QRect & source, int width, int height, int orientation );
+    static QRect rotateRect( const QRect & source, int width, int height, int orientation );
+
+    /**
+     * Return the horizontal DPI of the main display
+     */
+    static double getDpiX();
+
+    /**
+     * Return the vertical DPI of the main display
+     */
+    static double getDpiY();
 };
 
+}
 
 #endif
