@@ -23,7 +23,6 @@
 
 // qt/kde includes
 #include <qsplitter.h>
-#include <qpainter.h>
 #include <qlayout.h>
 #include <qlabel.h>
 #include <kvbox.h>
@@ -34,7 +33,6 @@
 #include <kinstance.h>
 #include <kprinter.h>
 #include <kstdaction.h>
-#include <kdeversion.h>
 #include <kparts/genericfactory.h>
 #include <kfiledialog.h>
 #include <kfind.h>
@@ -50,7 +48,7 @@
 #include <kprocess.h>
 #include <kservicetypetrader.h>
 #include <kstandarddirs.h>
-#include <ktempfile.h>
+#include <ktemporaryfile.h>
 #include <ktoggleaction.h>
 #include <ktogglefullscreenaction.h>
 #include <kio/job.h>
@@ -67,25 +65,24 @@
 #include "ui/embeddedfilesdialog.h"
 #include "ui/propertiesdialog.h"
 #include "ui/presentationwidget.h"
+#include "ui/pagesizelabel.h"
 #include "conf/preferencesdialog.h"
 #include "settings.h"
 #include "core/document.h"
 #include "core/generator.h"
 #include "core/page.h"
 
-typedef KParts::GenericFactory<okular::Part> okularPartFactory;
+typedef KParts::GenericFactory<Part> okularPartFactory;
 K_EXPORT_COMPONENT_FACTORY(libokularpart, okularPartFactory)
 
-using namespace okular;
-
 Part::Part(QWidget *parentWidget,
            QObject *parent,
            const QStringList & /*args*/ )
-	: KParts::ReadOnlyPart(parent), m_viewportDirty( 0 ),
+	: KParts::ReadOnlyPart(parent), 
 	m_showMenuBarAction(0), m_showFullScreenAction(0), m_actionsSearched(false),
 	m_searchStarted(false)
 {
-	QDBus::sessionBus().registerObject("/okular", this, QDBusConnection::ExportSlots);
+	QDBusConnection::sessionBus().registerObject("/okular", this, QDBusConnection::ExportScriptableSlots);
 
 	// connect the started signal to tell the job the mimetypes we like
 	connect(this, SIGNAL(started(KIO::Job *)), this, SLOT(setMimeTypes(KIO::Job *)));
@@ -99,12 +96,12 @@ Part::Part(QWidget *parentWidget,
 	setInstance(okularPartFactory::instance());
 
 	// build the document
-	m_document = new KPDFDocument(&m_loadedGenerators);
+	m_document = new Okular::Document(&m_loadedGenerators);
 	connect( m_document, SIGNAL( linkFind() ), this, SLOT( slotFind() ) );
 	connect( m_document, SIGNAL( linkGoToPage() ), this, SLOT( slotGoToPage() ) );
 	connect( m_document, SIGNAL( linkPresentation() ), this, SLOT( slotShowPresentation() ) );
 	connect( m_document, SIGNAL( linkEndPresentation() ), this, SLOT( slotHidePresentation() ) );
-	connect( m_document, SIGNAL( openURL(const KUrl &) ), this, SLOT( openURLFromDocument(const KUrl &) ) );
+	connect( m_document, SIGNAL( openUrl(const KUrl &) ), this, SLOT( openUrlFromDocument(const KUrl &) ) );
 	connect( m_document, SIGNAL( close() ), this, SLOT( close() ) );
 	
 	if ( parent && parent->metaObject()->indexOfSlot( SLOT( slotQuit() ) ) != -1 )
@@ -132,6 +129,7 @@ Part::Part(QWidget *parentWidget,
 	m_leftPanel->setMaximumWidth( 300 );
 	QVBoxLayout * leftPanelLayout = new QVBoxLayout( m_leftPanel );
 	leftPanelLayout->setMargin( 0 );
+	m_splitter->setCollapsible( 0, true );
 
 	// widgets: [left toolbox/..] | []
 	m_toolBox = new QToolBox( m_leftPanel );
@@ -151,8 +149,8 @@ Part::Part(QWidget *parentWidget,
 	m_searchWidget = new SearchWidget( thumbsBox, m_document );
 	m_thumbnailList = new ThumbnailList( thumbsBox, m_document );
 //	ThumbnailController * m_tc = new ThumbnailController( thumbsBox, m_thumbnailList );
-	connect( m_thumbnailList, SIGNAL( urlDropped( const KUrl& ) ), SLOT( openURLFromDocument( const KUrl & )) );
-	connect( m_thumbnailList, SIGNAL( rightClick(const KPDFPage *, const QPoint &) ), this, SLOT( slotShowMenu(const KPDFPage *, const QPoint &) ) );
+	connect( m_thumbnailList, SIGNAL( urlDropped( const KUrl& ) ), SLOT( openUrlFromDocument( const KUrl & )) );
+	connect( m_thumbnailList, SIGNAL( rightClick(const Okular::Page *, const QPoint &) ), this, SLOT( slotShowMenu(const Okular::Page *, const QPoint &) ) );
 	tbIndex = m_toolBox->addItem( thumbsBox, SmallIconSet("thumbnail"), i18n("Thumbnails") );
 	m_toolBox->setItemToolTip( tbIndex, i18n("Thumbnails") );
 	m_toolBox->setCurrentIndex( m_toolBox->indexOf( thumbsBox ) );
@@ -167,26 +165,47 @@ Part::Part(QWidget *parentWidget,
 	QVBoxLayout * miniBarLayout = new QVBoxLayout( miniBarContainer );
 	miniBarLayout->setMargin( 0 );
 	// widgets: [../[spacer/..]] | []
-	QWidget * miniSpacer = new QWidget( miniBarContainer );
-	miniSpacer->setFixedHeight( 6 );
-	miniBarLayout->addWidget( miniSpacer );
+	miniBarLayout->addItem( new QSpacerItem( 6, 6, QSizePolicy::Fixed, QSizePolicy::Fixed ) );
 	// widgets: [../[../MiniBar]] | []
-	m_miniBar = new MiniBar( miniBarContainer, m_document );
-	miniBarLayout->addWidget( m_miniBar );
+	QFrame * bevelContainer = new QFrame( miniBarContainer );
+	bevelContainer->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
+	QVBoxLayout * bevelContainerLayout = new QVBoxLayout( bevelContainer );
+	bevelContainerLayout->setMargin( 4 );
+	m_progressWidget = new ProgressWidget( bevelContainer, m_document );
+	bevelContainerLayout->addWidget( m_progressWidget );
+	miniBarLayout->addWidget( bevelContainer );
+	miniBarLayout->addItem( new QSpacerItem( 6, 6, QSizePolicy::Fixed, QSizePolicy::Fixed ) );
 
 	// widgets: [] | [right 'pageView']
-//	QWidget * rightContainer = new QWidget( m_splitter );
-//	QVBoxLayout * rightLayout = new QVBoxLayout( rightContainer );
+	QWidget * rightContainer = new QWidget( m_splitter );
+	QVBoxLayout * rightLayout = new QVBoxLayout( rightContainer );
+	rightLayout->setMargin( 0 );
+	rightLayout->setSpacing( 0 );
 //	KToolBar * rtb = new KToolBar( rightContainer, "mainToolBarSS" );
 //	rightLayout->addWidget( rtb );
-	m_pageView = new PageView( m_splitter, m_document );
+	m_topMessage = new PageViewTopMessage( rightContainer );
+	connect( m_topMessage, SIGNAL( action() ), this, SLOT( slotShowEmbeddedFiles() ) );
+	rightLayout->addWidget( m_topMessage );
+	m_pageView = new PageView( rightContainer, m_document );
 	m_pageView->setFocus(); //usability setting
-	connect( m_pageView, SIGNAL( urlDropped( const KUrl& ) ), SLOT( openURLFromDocument( const KUrl & )));
-	connect( m_pageView, SIGNAL( rightClick(const KPDFPage *, const QPoint &) ), this, SLOT( slotShowMenu(const KPDFPage *, const QPoint &) ) );
+	connect( m_pageView, SIGNAL( urlDropped( const KUrl& ) ), SLOT( openUrlFromDocument( const KUrl & )));
+	connect( m_pageView, SIGNAL( rightClick(const Okular::Page *, const QPoint &) ), this, SLOT( slotShowMenu(const Okular::Page *, const QPoint &) ) );
 	connect( m_document, SIGNAL( error( const QString&, int ) ), m_pageView, SLOT( errorMessage( const QString&, int ) ) );
 	connect( m_document, SIGNAL( warning( const QString&, int ) ), m_pageView, SLOT( warningMessage( const QString&, int ) ) );
 	connect( m_document, SIGNAL( notice( const QString&, int ) ), m_pageView, SLOT( noticeMessage( const QString&, int ) ) );
-//	rightLayout->addWidget( m_pageView );
+	rightLayout->addWidget( m_pageView );
+	QWidget * bottomBar = new QWidget( rightContainer );
+	QHBoxLayout * bottomBarLayout = new QHBoxLayout( bottomBar );
+	m_pageSizeLabel = new PageSizeLabel( bottomBar, m_document );
+	bottomBarLayout->setMargin( 0 );
+	bottomBarLayout->setSpacing( 0 );
+	bottomBarLayout->addWidget( m_pageSizeLabel->antiWidget() );
+	bottomBarLayout->addItem( new QSpacerItem( 5, 5, QSizePolicy::Expanding, QSizePolicy::Minimum ) );
+	m_miniBar = new MiniBar( bottomBar, m_document );
+	bottomBarLayout->addWidget( m_miniBar );
+	bottomBarLayout->addItem( new QSpacerItem( 5, 5, QSizePolicy::Expanding, QSizePolicy::Minimum ) );
+	bottomBarLayout->addWidget( m_pageSizeLabel );
+	rightLayout->addWidget( bottomBar );
 
 	// add document observers
 	m_document->addObserver( this );
@@ -194,7 +213,9 @@ Part::Part(QWidget *parentWidget,
 	m_document->addObserver( m_pageView );
 	m_document->addObserver( m_toc );
 	m_document->addObserver( m_miniBar );
+	m_document->addObserver( m_progressWidget );
 	m_document->addObserver( reviewsWidget );
+	m_document->addObserver( m_pageSizeLabel );
 
 	// ACTIONS
 	KActionCollection * ac = actionCollection();
@@ -210,12 +231,14 @@ Part::Part(QWidget *parentWidget,
 	m_prevPage->setShortcut( 0 );
 	// dirty way to activate prev page when pressing miniBar's button
 	connect( m_miniBar, SIGNAL( prevPage() ), m_prevPage, SLOT( trigger() ) );
+	connect( m_progressWidget, SIGNAL( prevPage() ), m_prevPage, SLOT( trigger() ) );
 
 	m_nextPage = KStdAction::next(this, SLOT(slotNextPage()), ac, "next_page" );
 	m_nextPage->setWhatsThis( i18n( "Moves to the next page of the document" ) );
 	m_nextPage->setShortcut( 0 );
 	// dirty way to activate next page when pressing miniBar's button
 	connect( m_miniBar, SIGNAL( nextPage() ), m_nextPage, SLOT( trigger() ) );
+	connect( m_progressWidget, SIGNAL( nextPage() ), m_nextPage, SLOT( trigger() ) );
 
 	m_firstPage = KStdAction::firstPage( this, SLOT( slotGotoFirst() ), ac, "first_page" );
 	m_firstPage->setWhatsThis( i18n( "Moves to the first page of the document" ) );
@@ -271,8 +294,8 @@ Part::Part(QWidget *parentWidget,
 	m_showLeftPanel = new KToggleAction( KIcon( "show_side_panel" ), i18n( "Show &Navigation Panel"), ac, "show_leftpanel" );
 	connect( m_showLeftPanel, SIGNAL( toggled( bool ) ), this, SLOT( slotShowLeftPanel() ) );
 	m_showLeftPanel->setShortcut( QKeySequence(Qt::CTRL + Qt::Key_L) );
-	m_showLeftPanel->setCheckedState( i18n( "Hide &Navigation Panel" ) );
-	m_showLeftPanel->setChecked( KpdfSettings::showLeftPanel() );
+	m_showLeftPanel->setCheckedState( KGuiItem(i18n( "Hide &Navigation Panel" ) ));
+	m_showLeftPanel->setChecked( Okular::Settings::showLeftPanel() );
 	slotShowLeftPanel();
 
 	KAction * importPS= new KAction(KIcon("psimport"), i18n("&Import Postscript as PDF..."), ac, "import_ps");
@@ -285,7 +308,7 @@ Part::Part(QWidget *parentWidget,
 	connect(m_showProperties, SIGNAL(triggered()), this, SLOT(slotShowProperties()));
 	m_showProperties->setEnabled( false );
 	
-	m_showEmbeddedFiles = new KAction(i18n("&Embedded Files"), ac, "embeddedFiles");
+	m_showEmbeddedFiles = new KAction(i18n("&Embedded Files"), ac, "embedded_files");
 	connect(m_showEmbeddedFiles, SIGNAL(triggered()), this, SLOT(slotShowEmbeddedFiles()));
 	m_showEmbeddedFiles->setEnabled( false );
 
@@ -305,7 +328,7 @@ Part::Part(QWidget *parentWidget,
 	m_pageView->setupActions( ac );
 
 	// apply configuration (both internal settings and GUI configured items)
-	QList<int> splitterSizes = KpdfSettings::splitterSizes();
+	QList<int> splitterSizes = Okular::Settings::splitterSizes();
 	if ( !splitterSizes.count() )
 	{
 		// the first time use 1/10 for the panel and 9/10 for the pageView
@@ -313,9 +336,8 @@ Part::Part(QWidget *parentWidget,
 		splitterSizes.push_back( 500 );
 	}
 	m_splitter->setSizes( splitterSizes );
-	// get notified about splitter size changes (HACK that will be removed
-	// by connecting to Qt4::QSplitter's sliderMoved())
-	m_pageView->installEventFilter( this );
+	// get notified about splitter size changes
+	connect( m_splitter, SIGNAL( splitterMoved( int, int ) ), this, SLOT( splitterMoved( int, int ) ) );
 
 	// document watcher and reloader
 	m_watcher = new KDirWatch( this );
@@ -328,8 +350,8 @@ Part::Part(QWidget *parentWidget,
 
 	// [SPEECH] check for KTTSD presence and usability
 	KService::List offers = KServiceTypeTrader::self()->query("DCOP/Text-to-Speech", "Name == 'KTTSD'");
-	KpdfSettings::setUseKTTSD( !offers.isEmpty() );
-	KpdfSettings::writeConfig();
+	Okular::Settings::setUseKTTSD( !offers.isEmpty() );
+	Okular::Settings::writeConfig();
 
 	// set our XML-UI resource file
 	setXMLFile("part.rc");
@@ -343,15 +365,17 @@ Part::~Part()
     delete m_pageView;
     delete m_thumbnailList;
     delete m_miniBar;
+    delete m_progressWidget;
+    delete m_pageSizeLabel;
 
     delete m_document;
 }
 
-void Part::openURLFromDocument(const KUrl &url)
+void Part::openUrlFromDocument(const KUrl &url)
 {
-    m_bExtension->openURLNotify();
-    m_bExtension->setLocationBarURL(url.prettyUrl());
-    openURL(url);
+    m_bExtension->openUrlNotify();
+    m_bExtension->setLocationBarUrl(url.prettyUrl());
+    openUrl(url);
 }
 
 void Part::supportedMimetypes()
@@ -400,7 +424,7 @@ void Part::fillGenerators()
         for (int i=0;i<count;i++)
         {
           propName=offers[i]->property("Name").toString();
-          // dont load already loaded generators
+          // don't load already loaded generators
           if (! m_loadedGenerators.take( propName ) )
           {
             KLibrary *lib = loader->globalLibrary( QFile::encodeName( offers[i]->library() ) );
@@ -410,7 +434,7 @@ void Part::fillGenerators()
 		continue;
             }
 
-            Generator* (*create_plugin)(KPDFDocument* doc) = ( Generator* (*)(KPDFDocument* doc) ) lib->symbol( "create_plugin" );
+            Okular::Generator* (*create_plugin)() = ( Okular::Generator* (*)() ) lib->symbol( "create_plugin" );
             if ( !create_plugin )
             {
                 kWarning() << "Library '" << offers.at(i)->library() << "' has no symbol 'create_plugin'." << endl;
@@ -418,7 +442,7 @@ void Part::fillGenerators()
             }
 
             // the generator should do anything with the document if we are only configuring
-            m_loadedGenerators.insert(propName,create_plugin(m_document));
+            m_loadedGenerators.insert(propName,create_plugin());
             m_generatorsWithSettings << propName;
           }
         }
@@ -436,17 +460,17 @@ void Part::slotGeneratorPreferences( )
         return;
 
     // we didn't find an instance of this dialog, so lets create it
-    KConfigDialog * dialog = new KConfigDialog( m_pageView, "generator_prefs", KpdfSettings::self() );
+    KConfigDialog * dialog = new KConfigDialog( m_pageView, "generator_prefs", Okular::Settings::self() );
     dialog->setCaption( i18n( "Configure Backends" ) );
 
-    QHashIterator<QString, Generator*> it(m_loadedGenerators);
+    QHashIterator<QString, Okular::Generator*> it(m_loadedGenerators);
     while(it.hasNext())
     {
         it.next();
         it.value()->addPages(dialog);
     }
 
-    // (for now dont FIXME) keep us informed when the user changes settings
+    // (for now don't FIXME) keep us informed when the user changes settings
     // connect( dialog, SIGNAL( settingsChanged() ), this, SLOT( slotNewConfig() ) );
     dialog->show();
 }
@@ -471,7 +495,7 @@ void Part::goToPage(uint i)
 
 void Part::openDocument(KUrl doc)
 {
-    openURL(doc);
+    openUrl(doc);
 }
 
 uint Part::pages()
@@ -511,13 +535,17 @@ bool Part::slotImportPSFile()
 	}
 
     KUrl url = KFileDialog::getOpenUrl( KUrl(), "application/postscript", this->widget() );
-    KTempFile tf( QString::null, ".pdf" );
-
-    if ( tf.status() == 0 && url.isLocalFile())
+    if ( url.isLocalFile() )
     {
+        KTemporaryFile tf;
+        tf.setSuffix( ".pdf" );
+        tf.setAutoRemove( false );
+        if ( !tf.open() )
+            return false;
+        m_temporaryLocalFile = tf.fileName();
         tf.close();
+
         m_file = url.path();
-        m_temporaryLocalFile = tf.name();
         KProcess *p = new KProcess;
         *p << app;
         *p << m_file << m_temporaryLocalFile;
@@ -551,24 +579,29 @@ bool Part::openFile()
     m_saveAs->setEnabled( ok );
     m_printPreview->setEnabled( ok );
     m_showProperties->setEnabled( ok );
-    m_showEmbeddedFiles->setEnabled( ok && m_document->embeddedFiles() && m_document->embeddedFiles()->count() > 0);
+    bool hasEmbeddedFiles = ok && m_document->embeddedFiles() && m_document->embeddedFiles()->count() > 0;
+    m_showEmbeddedFiles->setEnabled( hasEmbeddedFiles );
+    if ( hasEmbeddedFiles )
+        m_topMessage->display( i18n( "This document has embedded files. <a href=\"okular:/embeddedfiles\">Click here to see them</a> or go to File -> Embedded Files." ) );
+    else
+        m_topMessage->hide();
     m_showPresentation->setEnabled( ok );
     if ( ok )
     {
-        m_exportItems = m_document->exportFormats();
-        QList<ExportEntry*>::ConstIterator it = m_exportItems.constBegin();
-        QList<ExportEntry*>::ConstIterator itEnd = m_exportItems.constEnd();
+        m_exportFormats = m_document->exportFormats();
+        QList<Okular::ExportFormat>::ConstIterator it = m_exportFormats.constBegin();
+        QList<Okular::ExportFormat>::ConstIterator itEnd = m_exportFormats.constEnd();
         QMenu *menu = m_exportAs->menu();
         for ( ; it != itEnd; ++it )
         {
-            ExportEntry* cur = *it;
-            if ( !cur->icon.isEmpty() )
+            const Okular::ExportFormat &format = *it;
+            if ( !format.icon().isNull() )
             {
-                menu->addAction( KIcon( cur->icon ), cur->description );
+                menu->addAction( format.icon(), format.description() );
             }
             else
             {
-                menu->addAction( cur->description );
+                menu->addAction( format.description() );
             }
         }
     }
@@ -580,7 +613,7 @@ bool Part::openFile()
     if ( !ok )
     {
         // if can't open document, update windows so they display blank contents
-        m_pageView->updateContents();
+        m_pageView->widget()->update();
         m_thumbnailList->update();
         return false;
     }
@@ -589,8 +622,13 @@ bool Part::openFile()
     if ( !m_watcher->contains(m_file) )
         m_watcher->addFile(m_file);
 
+    // if the 'OpenTOC' flag is set, start presentation
+    if ( m_document->getMetaData( "OpenTOC" ).toBool() && m_toolBox->isItemEnabled( 0 ) )
+    {
+        m_toolBox->setCurrentIndex( 0 );
+    }
     // if the 'StartFullScreen' flag is set, start presentation
-    if ( m_document->getMetaData( "StartFullScreen" ) == "yes" )
+    if ( m_document->getMetaData( "StartFullScreen" ).toBool() )
     {
         KMessageBox::information( m_presentationWidget, i18n("The document is going to be launched on presentation mode because the file requested it."), QString::null, "autoPresentationWarning" );
         slotShowPresentation();
@@ -601,25 +639,29 @@ bool Part::openFile()
     return true;
 }
 
-bool Part::openURL(const KUrl &url)
+bool Part::openUrl(const KUrl &url)
 {
     // note: this can be the right place to check the file for gz or bz2 extension
     // if it matches then: download it (if not local) extract to a temp file using
     // KTar and proceed with the URL of the temporary file
 
-    // this calls in sequence the 'closeURL' and 'openFile' methods
-    bool openOk = KParts::ReadOnlyPart::openURL(url);
+    // this calls in sequence the 'closeUrl' and 'openFile' methods
+    bool openOk = KParts::ReadOnlyPart::openUrl(url);
 
     if ( openOk )
     {
-        m_viewportDirty = 0;
+        m_viewportDirty.pageNumber = -1;
 
         // if the document have a 'DocumentTitle' flag set (and it is not empty), set it as title
-        QString title = m_document->getMetaData( "DocumentTitle" );
+        QString title = m_document->getMetaData( "DocumentTitle" ).toString();
         if ( !title.isEmpty() && !title.trimmed().isEmpty() )
         {
             emit setWindowCaption( title );
         }
+        else
+        {
+            emit setWindowCaption( url.fileName() );
+        }
     }
     else
         KMessageBox::error( widget(), i18n( "Could not open %1", url.prettyUrl() ) );
@@ -627,7 +669,7 @@ bool Part::openURL(const KUrl &url)
     return openOk;
 }
 
-bool Part::closeURL()
+bool Part::closeUrl()
 {
     if (!m_temporaryLocalFile.isNull())
     {
@@ -643,7 +685,7 @@ bool Part::closeURL()
     m_showProperties->setEnabled( false );
     m_showEmbeddedFiles->setEnabled( false );
     m_exportAsText->setEnabled( false );
-    m_exportItems.clear();
+    m_exportFormats.clear();
     QMenu *menu = m_exportAs->menu();
     QList<QAction*> acts = menu->actions();
     int num = acts.count();
@@ -660,38 +702,35 @@ bool Part::closeURL()
     m_document->closeDocument();
     updateViewActions();
     m_searchWidget->clearText();
-    return KParts::ReadOnlyPart::closeURL();
+    return KParts::ReadOnlyPart::closeUrl();
 }
 
 void Part::close()
 {
   if (parent() && (parent()->objectName() == QLatin1String("okular::Shell")))
   {
-    closeURL();
+    closeUrl();
   }
-  else KMessageBox::information(widget(), i18n("This link points to a close document action that does not work when using the embedded viewer."), QString::null, "warnNoCloseIfNotInKPDF");
+  else KMessageBox::information( widget(), i18n( "This link points to a close document action that does not work when using the embedded viewer." ), QString::null, "warnNoCloseIfNotInOkular" );
 }
 
 void Part::cannotQuit()
 {
-	KMessageBox::information(widget(), i18n("This link points to a quit application action that does not work when using the embedded viewer."), QString::null, "warnNoQuitIfNotInKPDF");
+    KMessageBox::information( widget(), i18n( "This link points to a quit application action that does not work when using the embedded viewer." ), QString::null, "warnNoQuitIfNotInOkular" );
 }
 
-bool Part::eventFilter( QObject * watched, QEvent * e )
+void Part::splitterMoved( int /*pos*/, int index )
 {
     // if pageView has been resized, save splitter sizes
-    if ( watched == m_pageView && e->type() == QEvent::Resize )
+    if ( index == 1 )
         saveSplitterSize();
-
-    // only intercept events, don't block them
-    return false;
 }
 
 void Part::slotShowLeftPanel()
 {
     bool showLeft = m_showLeftPanel->isChecked();
-    KpdfSettings::setShowLeftPanel( showLeft );
-    KpdfSettings::writeConfig();
+    Okular::Settings::setShowLeftPanel( showLeft );
+    Okular::Settings::writeConfig();
     // show/hide left panel
     m_leftPanel->setVisible( showLeft );
     // this needs to be hidden explicitly to disable thumbnails gen
@@ -700,8 +739,8 @@ void Part::slotShowLeftPanel()
 
 void Part::saveSplitterSize()
 {
-    KpdfSettings::setSplitterSizes( m_splitter->sizes() );
-    KpdfSettings::writeConfig();
+    Okular::Settings::setSplitterSizes( m_splitter->sizes() );
+    Okular::Settings::writeConfig();
 }
 
 void Part::slotFileDirty( const QString& fileName )
@@ -723,20 +762,24 @@ void Part::slotDoFileDirty()
     if ( m_viewportDirty.pageNumber == -1 )
     {
         // store the current viewport
-        m_viewportDirty = DocumentViewport( m_document->viewport() );
+        m_viewportDirty = m_document->viewport();
+
+        // store if presentation view was open
+        m_wasPresentationOpen = ((PresentationWidget*)m_presentationWidget != 0);
 
         // inform the user about the operation in progress
         m_pageView->displayMessage( i18n("Reloading the document...") );
     }
 
     // close and (try to) reopen the document
-    if ( KParts::ReadOnlyPart::openURL(m_file) )
+    if ( KParts::ReadOnlyPart::openUrl(m_file) )
     {
         // on successfull opening, restore the previous viewport
         if ( m_viewportDirty.pageNumber >= (int) m_document->pages() ) 
             m_viewportDirty.pageNumber = (int) m_document->pages() - 1;
         m_document->setViewport( m_viewportDirty );
         m_viewportDirty.pageNumber = -1;
+        if (m_wasPresentationOpen) slotShowPresentation();
         emit enablePrintAction(true);
     }
     else
@@ -780,10 +823,10 @@ void Part::enableTOC(bool enable)
 }
 
 //BEGIN go to page dialog
-class KPDFGotoPageDialog : public KDialog
+class GotoPageDialog : public KDialog
 {
 public:
-	KPDFGotoPageDialog(QWidget *p, int current, int max) : KDialog(p) {
+	  GotoPageDialog(QWidget *p, int current, int max) : KDialog(p) {
 		setCaption(i18n("Go to Page"));
 		setButtons(Ok | Cancel);
 		setDefaultButton(Ok);
@@ -818,7 +861,7 @@ public:
 
 void Part::slotGoToPage()
 {
-    KPDFGotoPageDialog pageDialog( m_pageView, m_document->currentPage() + 1, m_document->pages() );
+    GotoPageDialog pageDialog( m_pageView, m_document->currentPage() + 1, m_document->pages() );
     if ( pageDialog.exec() == QDialog::Accepted )
         m_document->setViewportPage( pageDialog.getPage() - 1 );
 }
@@ -907,7 +950,7 @@ void Part::slotFind()
         m_searchStarted = true;
         m_document->resetSearch( PART_SEARCH_ID );
         m_document->searchText( PART_SEARCH_ID, dlg.pattern(), false, dlg.options() & KFind::CaseSensitive,
-                                KPDFDocument::NextMatch, true, qRgb( 255, 255, 64 ) );
+                                Okular::Document::NextMatch, true, qRgb( 255, 255, 64 ) );
     }
 }
 
@@ -919,17 +962,17 @@ void Part::slotFindNext()
 
 void Part::slotSaveFileAs()
 {
-    KUrl saveURL = KFileDialog::getSaveUrl( url().isLocalFile() ? url().url() : url().fileName(), QString::null, widget() );
-    if ( saveURL.isValid() && !saveURL.isEmpty() )
+    KUrl saveUrl = KFileDialog::getSaveUrl( url().isLocalFile() ? url().url() : url().fileName(), QString::null, widget() );
+    if ( saveUrl.isValid() && !saveUrl.isEmpty() )
     {
-        if ( KIO::NetAccess::exists( saveURL, false, widget() ) )
+        if ( KIO::NetAccess::exists( saveUrl, false, widget() ) )
         {
-            if (KMessageBox::warningContinueCancel( widget(), i18n("A file named \"%1\" already exists. Are you sure you want to overwrite it?", saveURL.fileName()), QString::null, i18n("Overwrite")) != KMessageBox::Continue)
+            if (KMessageBox::warningContinueCancel( widget(), i18n("A file named \"%1\" already exists. Are you sure you want to overwrite it?", saveUrl.fileName()), QString::null, KGuiItem(i18n("Overwrite"))) != KMessageBox::Continue)
                 return;
         }
 
-        if ( !KIO::NetAccess::file_copy( url(), saveURL, -1, true ) )
-            KMessageBox::information( widget(), i18n("File could not be saved in '%1'. Try to save it to another location.", saveURL.prettyUrl() ) );
+        if ( !KIO::NetAccess::file_copy( url(), saveUrl, -1, true ) )
+            KMessageBox::information( widget(), i18n("File could not be saved in '%1'. Try to save it to another location.", saveUrl.prettyUrl() ) );
     }
 }
 
@@ -949,7 +992,7 @@ void Part::slotPreferences()
         return;
 
     // we didn't find an instance of this dialog, so lets create it
-    PreferencesDialog * dialog = new PreferencesDialog( m_pageView, KpdfSettings::self() );
+    PreferencesDialog * dialog = new PreferencesDialog( m_pageView, Okular::Settings::self() );
     // keep us informed when the user changes settings
     connect( dialog, SIGNAL( settingsChanged( const QString & ) ), this, SLOT( slotNewConfig() ) );
 
@@ -962,7 +1005,7 @@ void Part::slotNewConfig()
     // changed before applying changes.
 
     // Watch File
-    bool watchFile = KpdfSettings::watchFile();
+    bool watchFile = Okular::Settings::watchFile();
     if ( watchFile && m_watcher->isStopped() )
         m_watcher->startScan();
     if ( !watchFile && !m_watcher->isStopped() )
@@ -971,26 +1014,18 @@ void Part::slotNewConfig()
         m_watcher->stopScan();
     }
 
-    bool showSearch = KpdfSettings::showSearchBar();
+    bool showSearch = Okular::Settings::showSearchBar();
     if ( !m_searchWidget->isHidden() != showSearch )
         m_searchWidget->setVisible( showSearch );
 
     // Main View (pageView)
-    Q3ScrollView::ScrollBarMode scrollBarMode = KpdfSettings::showScrollBars() ?
-        Q3ScrollView::AlwaysOn : Q3ScrollView::AlwaysOff;
-    if ( m_pageView->hScrollBarMode() != scrollBarMode )
-    {
-        m_pageView->setHScrollBarMode( scrollBarMode );
-        m_pageView->setVScrollBarMode( scrollBarMode );
-    }
+    m_pageView->reparseConfig();
 
     // update document settings
     m_document->reparseConfig();
 
-    // update Main View and ThumbnailList contents
-    // TODO do this only when changing KpdfSettings::renderMode()
-    m_pageView->updateContents();
-    if ( KpdfSettings::showLeftPanel() && !m_thumbnailList->isHidden() )
+    // update ThumbnailList contents
+    if ( Okular::Settings::showLeftPanel() && !m_thumbnailList->isHidden() )
         m_thumbnailList->updateWidgets();
 }
 
@@ -1001,7 +1036,7 @@ void Part::slotPrintPreview()
     double width, height;
     int landscape, portrait;
     KPrinter printer;
-    const KPDFPage *page;
+    const Okular::Page *page;
 
     printer.setMinMax(1, m_document->pages());
     printer.setPreviewOnly( true );
@@ -1020,12 +1055,16 @@ void Part::slotPrintPreview()
         if (width > height) landscape++;
         else portrait++;
     }
-    if (landscape > portrait) printer.setOption("orientation-requested", "4");
+    if (landscape > portrait)
+    {
+    //  printer.setOption("orientation-requested", "4");
+        printer.setOrientation(KPrinter::Landscape);
+    }
 
     doPrint(printer);
 }
 
-void Part::slotShowMenu(const KPDFPage *page, const QPoint &point)
+void Part::slotShowMenu(const Okular::Page *page, const QPoint &point)
 {
 	bool reallyShow = false;
 	if (!m_actionsSearched)
@@ -1137,11 +1176,11 @@ void Part::slotExportAs(QAction * act)
     if ( ( id < 0 ) || ( id >= acts.count() ) )
         return;
 
-    QString filter = id == 0 ? "text/plain" : m_exportItems.at( id - 1 )->mime->name();
+    QString filter = id == 0 ? "text/plain" : m_exportFormats.at( id - 1 ).mimeType()->name();
     QString fileName = KFileDialog::getSaveFileName( url().isLocalFile() ? url().fileName() : QString::null, filter, widget() );
     if ( !fileName.isEmpty() )
     {
-        bool saved = id == 0 ? m_document->exportToText( fileName ) : m_document->exportTo( fileName, m_exportItems.at( id - 1 )->mime );
+        bool saved = id == 0 ? m_document->exportToText( fileName ) : m_document->exportTo( fileName, m_exportFormats.at( id - 1 ) );
         if ( !saved )
             KMessageBox::information( widget(), i18n("File could not be saved in '%1'. Try to save it to another location.", fileName ) );
     }
@@ -1154,7 +1193,7 @@ void Part::slotPrint()
     double width, height;
     int landscape, portrait;
     KPrinter printer;
-    const KPDFPage *page;
+    const Okular::Page *page;
 
     printer.setPageSelection(KPrinter::ApplicationSide);
     printer.setMinMax(1, m_document->pages());
@@ -1183,7 +1222,7 @@ void Part::slotPrint()
 
 void Part::doPrint(KPrinter &printer)
 {
-    if (!m_document->isAllowed(KPDFDocument::AllowPrint))
+    if (!m_document->isAllowed(Okular::Document::AllowPrint))
     {
         KMessageBox::error(widget(), i18n("Printing this document is not allowed."));
         return;
@@ -1195,15 +1234,21 @@ void Part::doPrint(KPrinter &printer)
     }
 }
 
-void Part::restoreDocument(const KUrl &url, int page)
+void Part::restoreDocument(KConfig* config)
 {
-  if (openURL(url)) goToPage(page);
+  KUrl url ( config->readPathEntry( "URL" ) );
+  if ( url.isValid() )
+  {
+    QString viewport = config->readEntry( "Viewport" );
+    if (!viewport.isEmpty()) m_document->setNextDocumentViewport( Okular::DocumentViewport( viewport ) );
+    openUrl( url );
+  }
 }
 
 void Part::saveDocumentRestoreInfo(KConfig* config)
 {
   config->writePathEntry( "URL", url().url() );
-  if (m_document->pages() > 0) config->writeEntry( "Page", m_document->currentPage() + 1 );
+  config->writeEntry( "Viewport", m_document->viewport().toString() );
 }
 
 void Part::psTransformEnded()
