@@ -12,6 +12,7 @@
 #include <qpainter.h>
 #include <qpixmap.h>
 #include <qimage.h>
+#include <qvarlengtharray.h>
 #include <qapplication.h>
 #include <kglobal.h>
 #include <kimageeffect.h>
@@ -32,7 +33,9 @@
 static KStaticDeleter<QPixmap> sd;
 QPixmap * busyPixmap = 0;
 
-void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * page,
+#define TEXTANNOTATION_ICONSIZE 24
+
+void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page * page,
     int pixID, int flags, int scaledWidth, int scaledHeight, const QRect & limits )
 {
     /** 1 - RETRIEVE THE 'PAGE+ID' PIXMAP OR A SIMILAR 'PAGE' ONE **/
@@ -65,9 +68,9 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
     if ( !pixmap || pixmapRescaleRatio > 20.0 || pixmapRescaleRatio < 0.25 ||
          (scaledWidth != pixmap->width() && pixmapPixels > 6000000L) )
     {
-        if ( KpdfSettings::changeColors() &&
-             KpdfSettings::renderMode() == KpdfSettings::EnumRenderMode::Paper )
-            destPainter->fillRect( limits, KpdfSettings::paperColor() );
+        if ( Okular::Settings::changeColors() &&
+             Okular::Settings::renderMode() == Okular::Settings::EnumRenderMode::Paper )
+            destPainter->fillRect( limits, Okular::Settings::paperColor() );
         else
             destPainter->fillRect( limits, Qt::white );
 
@@ -92,17 +95,18 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
 
     /** 2 - FIND OUT WHAT TO PAINT (Flags + Configuration + Presence) **/
     bool canDrawHighlights = (flags & Highlights) && !page->m_highlights.isEmpty();
+    bool canDrawTextSelection = (flags & TextSelection) && page->m_textSelections;
     bool canDrawAnnotations = (flags & Annotations) && !page->m_annotations.isEmpty();
-    bool enhanceLinks = (flags & EnhanceLinks) && KpdfSettings::highlightLinks();
-    bool enhanceImages = (flags & EnhanceImages) && KpdfSettings::highlightImages();
+    bool enhanceLinks = (flags & EnhanceLinks) && Okular::Settings::highlightLinks();
+    bool enhanceImages = (flags & EnhanceImages) && Okular::Settings::highlightImages();
     // vectors containing objects to draw
-    // make this a qcolor, rect map, since we dont need 
+    // make this a qcolor, rect map, since we don't need
     // to know s_id here! we are only drawing this right?
-    QList< QPair<QColor, NormalizedRect *> > * bufferedHighlights = 0;
-    QList< Annotation * > * bufferedAnnotations = 0;
-    QList< Annotation * > * unbufferedAnnotations = 0;
-    // fill up lists with visible annotation/highlight objects
-    if ( canDrawHighlights || canDrawAnnotations )
+    QList< QPair<QColor, Okular::NormalizedRect *> > * bufferedHighlights = 0;
+    QList< Okular::Annotation * > * bufferedAnnotations = 0;
+    QList< Okular::Annotation * > * unbufferedAnnotations = 0;
+    // fill up lists with visible annotation/highlight objects/text selections
+    if ( canDrawHighlights || canDrawTextSelection || canDrawAnnotations )
     {
         // precalc normalized 'limits rect' for intersection
         double nXMin = (double)limits.left() / (double)scaledWidth,
@@ -113,13 +117,13 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
         if ( canDrawHighlights )
         {
             if ( !bufferedHighlights )
-                 bufferedHighlights = new QList< QPair<QColor, NormalizedRect *>  >();
+                 bufferedHighlights = new QList< QPair<QColor, Okular::NormalizedRect *>  >();
 /*            else
             {*/
                 
-                NormalizedRect* limitRect = new NormalizedRect(nXMin, nYMin, nXMax, nYMax );
-                QLinkedList< HighlightAreaRect * >::const_iterator h2It = page->m_highlights.begin(), hEnd = page->m_highlights.end();
-                QList< NormalizedRect * >::const_iterator hIt;
+                Okular::NormalizedRect* limitRect = new Okular::NormalizedRect(nXMin, nYMin, nXMax, nYMax );
+                QLinkedList< Okular::HighlightAreaRect * >::const_iterator h2It = page->m_highlights.begin(), hEnd = page->m_highlights.end();
+                QList< Okular::NormalizedRect * >::const_iterator hIt;
                 for ( ; h2It != hEnd; ++h2It )
                     for (hIt=(*h2It)->begin(); hIt!=(*h2It)->end(); ++hIt)
                     {
@@ -129,27 +133,56 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
                 delete limitRect;
             //}
         }
+        if ( canDrawTextSelection )
+        {
+            if ( !bufferedHighlights )
+                 bufferedHighlights = new QList< QPair<QColor, Okular::NormalizedRect *>  >();
+/*            else
+            {*/
+                Okular::NormalizedRect* limitRect = new Okular::NormalizedRect(nXMin, nYMin, nXMax, nYMax );
+                QList< Okular::NormalizedRect * >::const_iterator hIt = page->m_textSelections->begin(), hEnd = page->m_textSelections->end();
+                for ( ; hIt != hEnd; ++hIt )
+                {
+                    if ( (*hIt)->intersects( limitRect ) )
+                        bufferedHighlights->append( qMakePair( page->m_textSelections->color, *hIt ) );
+                }
+                delete limitRect;
+            //}
+        }
         // append annotations inside limits to the un/buffered list
         if ( canDrawAnnotations )
         {
-            QLinkedList< Annotation * >::const_iterator aIt = page->m_annotations.begin(), aEnd = page->m_annotations.end();
+            QLinkedList< Okular::Annotation * >::const_iterator aIt = page->m_annotations.begin(), aEnd = page->m_annotations.end();
             for ( ; aIt != aEnd; ++aIt )
             {
-                Annotation * ann = *aIt;
-                if ( ann->boundary.intersects( nXMin, nYMin, nXMax, nYMax ) )
+                Okular::Annotation * ann = *aIt;
+                if ( ann->flags & Okular::Annotation::Hidden )
+                    continue;
+
+                bool intersects = ann->boundary.intersects( nXMin, nYMin, nXMax, nYMax );
+                if ( ann->subType() == Okular::Annotation::AText )
                 {
-                    Annotation::SubType type = ann->subType();
-                    if ( type == Annotation::ALine || type == Annotation::AHighlight ||
-                         type == Annotation::AInk  /*|| (type == Annotation::AGeom && ann->style.opacity < 0.99)*/ )
+                    Okular::TextAnnotation * ta = static_cast< Okular::TextAnnotation * >( ann );
+                    if ( ta->textType == Okular::TextAnnotation::Linked )
+                    {
+                        Okular::NormalizedRect iconrect( ann->boundary.left, ann->boundary.top, ann->boundary.left + TEXTANNOTATION_ICONSIZE / page->width(), ann->boundary.top + TEXTANNOTATION_ICONSIZE / page->height() );
+                        intersects = iconrect.intersects( nXMin, nYMin, nXMax, nYMax );
+                    }
+                }
+                if ( intersects )
+                {
+                    Okular::Annotation::SubType type = ann->subType();
+                    if ( type == Okular::Annotation::ALine || type == Okular::Annotation::AHighlight ||
+                         type == Okular::Annotation::AInk  /*|| (type == Annotation::AGeom && ann->style.opacity < 0.99)*/ )
                     {
                         if ( !bufferedAnnotations )
-                            bufferedAnnotations = new QList< Annotation * >();
+                            bufferedAnnotations = new QList< Okular::Annotation * >();
                         bufferedAnnotations->append( ann );
                     }
                     else
                     {
                         if ( !unbufferedAnnotations )
-                            unbufferedAnnotations = new QList< Annotation * >();
+                            unbufferedAnnotations = new QList< Okular::Annotation * >();
                         unbufferedAnnotations->append( ann );
                     }
                 }
@@ -159,7 +192,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
     }
 
     /** 3 - ENABLE BACKBUFFERING IF DIRECT IMAGE MANIPULATION IS NEEDED **/
-    bool bufferAccessibility = (flags & Accessibility) && KpdfSettings::changeColors() && (KpdfSettings::renderMode() != KpdfSettings::EnumRenderMode::Paper);
+    bool bufferAccessibility = (flags & Accessibility) && Okular::Settings::changeColors() && (Okular::Settings::renderMode() != Okular::Settings::EnumRenderMode::Paper);
     bool useBackBuffer = bufferAccessibility || bufferedHighlights || bufferedAnnotations;
     QPixmap * backPixmap = 0;
     QPainter * mixedPainter = 0;
@@ -197,21 +230,21 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
         // 4B.2. modify pixmap following accessibility settings
         if ( bufferAccessibility )
         {
-            switch ( KpdfSettings::renderMode() )
+            switch ( Okular::Settings::renderMode() )
             {
-                case KpdfSettings::EnumRenderMode::Inverted:
+                case Okular::Settings::EnumRenderMode::Inverted:
                     // Invert image pixels using QImage internal function
                     backImage.invertPixels(QImage::InvertRgb);
                     break;
-                case KpdfSettings::EnumRenderMode::Recolor:
+                case Okular::Settings::EnumRenderMode::Recolor:
                     // Recolor image using KImageEffect::flatten with dither:0
-                    KImageEffect::flatten( backImage, KpdfSettings::recolorForeground(), KpdfSettings::recolorBackground() );
+                    KImageEffect::flatten( backImage, Okular::Settings::recolorForeground(), Okular::Settings::recolorBackground() );
                     break;
-                case KpdfSettings::EnumRenderMode::BlackWhite:
+                case Okular::Settings::EnumRenderMode::BlackWhite:
                     // Manual Gray and Contrast
                     unsigned int * data = (unsigned int *)backImage.bits();
                     int val, pixels = backImage.width() * backImage.height(),
-                        con = KpdfSettings::bWContrast(), thr = 255 - KpdfSettings::bWThreshold();
+                        con = Okular::Settings::bWContrast(), thr = 255 - Okular::Settings::bWThreshold();
                     for( int i = 0; i < pixels; ++i )
                     {
                         val = qGray( data[i] );
@@ -236,10 +269,10 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
         if ( bufferedHighlights )
         {
             // draw highlights that are inside the 'limits' paint region
-            QList< QPair<QColor, NormalizedRect *> >::const_iterator hIt = bufferedHighlights->begin(), hEnd = bufferedHighlights->end();
+            QList< QPair<QColor, Okular::NormalizedRect *> >::const_iterator hIt = bufferedHighlights->begin(), hEnd = bufferedHighlights->end();
             for ( ; hIt != hEnd; ++hIt )
             {
-                NormalizedRect * r = (*hIt).second;
+                Okular::NormalizedRect * r = (*hIt).second;
                 // find out the rect to highlight on pixmap
                 QRect highlightRect = r->geometry( scaledWidth, scaledHeight ).intersect( limits );
                 highlightRect.translate( -limits.left(), -limits.top() );
@@ -276,63 +309,98 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
                    yScale = (double)scaledHeight / (double)limits.height();
 
             // paint all buffered annotations in the page
-            QList< Annotation * >::const_iterator aIt = bufferedAnnotations->begin(), aEnd = bufferedAnnotations->end();
+            QList< Okular::Annotation * >::const_iterator aIt = bufferedAnnotations->begin(), aEnd = bufferedAnnotations->end();
             for ( ; aIt != aEnd; ++aIt )
             {
-                Annotation * a = *aIt;
-                Annotation::SubType type = a->subType();
+                Okular::Annotation * a = *aIt;
+                Okular::Annotation::SubType type = a->subType();
 
-                // draw TextAnnotation (InPlace) MISSING: all
-                if ( type == Annotation::AText )
-                {
-                    // TODO
-                }
                 // draw LineAnnotation MISSING: all
-                else if ( type == Annotation::ALine )
+                if ( type == Okular::Annotation::ALine )
                 {
                     // get the annotation
-                    LineAnnotation * la = (LineAnnotation *) a;
+                    Okular::LineAnnotation * la = (Okular::LineAnnotation *) a;
 
                     NormalizedPath path;
                     // normalize page point to image
-					
-                    NormalizedPoint inkPoint = la->linePoints.first();
-                    NormalizedPoint point;
-                    point.x = (inkPoint.x - xOffset) * xScale;
-                    point.y = (inkPoint.y - yOffset) * yScale;
-                    path.append( point );
-					inkPoint = la->linePoints.last();
-					
-                    point.x = (inkPoint.x - xOffset) * xScale;
-                    point.y = (inkPoint.y - yOffset) * yScale;
-                    path.append( point );
-				
-                    // draw the line as normalized path into image
-                    drawShapeOnImage( backImage, path, false, QPen( a->style.color,a->style.width ), QBrush(), pageScale ,Multiply);
+                    QLinkedList<Okular::NormalizedPoint>::const_iterator it = la->linePoints.begin();
+                    QLinkedList<Okular::NormalizedPoint>::const_iterator itEnd = la->linePoints.end();
+                    for ( ; it != itEnd; ++it )
+                    {
+                        Okular::NormalizedPoint point;
+                        point.x = ( (*it).x - xOffset) * xScale;
+                        point.y = ( (*it).y - yOffset) * yScale;
+                        path.append( point );
+                    }
 
+                    // draw the line as normalized path into image
+                    drawShapeOnImage( backImage, path, la->lineClosed, QPen( a->style.color,a->style.width ), QBrush(), pageScale ,Multiply);
+
+                    if ( path.count() == 2 && fabs( la->lineLeadingFwdPt ) > 0.1 )
+                    {
+                        Okular::NormalizedPoint delta( la->linePoints.last().x - la->linePoints.first().x, la->linePoints.first().y - la->linePoints.last().y );
+                        double angle = atan2( delta.y, delta.x );
+                        if ( delta.y < 0 )
+                            angle += 2 * M_PI;
+
+                        int sign = la->lineLeadingFwdPt > 0.0 ? 1 : -1;
+                        double LLx = fabs( la->lineLeadingFwdPt ) * cos( angle + sign * M_PI_2 + 2 * M_PI ) / page->width();
+                        double LLy = fabs( la->lineLeadingFwdPt ) * sin( angle + sign * M_PI_2 + 2 * M_PI ) / page->height();
+
+                        NormalizedPath path2;
+                        NormalizedPath path3;
+
+                        Okular::NormalizedPoint point;
+                        point.x = ( la->linePoints.first().x + LLx - xOffset ) * xScale;
+                        point.y = ( la->linePoints.first().y - LLy - yOffset ) * yScale;
+                        path2.append( point );
+                        point.x = ( la->linePoints.last().x + LLx - xOffset ) * xScale;
+                        point.y = ( la->linePoints.last().y - LLy - yOffset ) * yScale;
+                        path3.append( point );
+                        // do we have the extension on the "back"?
+                        if ( fabs( la->lineLeadingBackPt ) > 0.1 )
+                        {
+                            double LLEx = la->lineLeadingBackPt * cos( angle - sign * M_PI_2 + 2 * M_PI ) / page->width();
+                            double LLEy = la->lineLeadingBackPt * sin( angle - sign * M_PI_2 + 2 * M_PI ) / page->height();
+                            point.x = ( la->linePoints.first().x + LLEx - xOffset ) * xScale;
+                            point.y = ( la->linePoints.first().y - LLEy - yOffset ) * yScale;
+                            path2.append( point );
+                            point.x = ( la->linePoints.last().x + LLEx - xOffset ) * xScale;
+                            point.y = ( la->linePoints.last().y - LLEy - yOffset ) * yScale;
+                            path3.append( point );
+                        }
+                        else
+                        {
+                            path2.append( path[0] );
+                            path3.append( path[1] );
+                        }
+
+                        drawShapeOnImage( backImage, path2, false, QPen( a->style.color, a->style.width ), QBrush(), pageScale, Multiply );
+                        drawShapeOnImage( backImage, path3, false, QPen( a->style.color, a->style.width ), QBrush(), pageScale, Multiply );
+                    }
                 }
                 // draw GeomAnnotation MISSING: all
-                else if ( type == Annotation::AGeom )
+                else if ( type == Okular::Annotation::AGeom )
                 {
                     // TODO
                 }
                 // draw HighlightAnnotation MISSING: under/strike width, feather, capping
-                else if ( type == Annotation::AHighlight )
+                else if ( type == Okular::Annotation::AHighlight )
                 {
                     // get the annotation
-                    HighlightAnnotation * ha = (HighlightAnnotation *) a;
-                    HighlightAnnotation::HighlightType type = ha->highlightType;
+                    Okular::HighlightAnnotation * ha = (Okular::HighlightAnnotation *) a;
+                    Okular::HighlightAnnotation::HighlightType type = ha->highlightType;
 
                     // draw each quad of the annotation
                     int quads = ha->highlightQuads.size();
                     for ( int q = 0; q < quads; q++ )
                     {
                         NormalizedPath path;
-                        const HighlightAnnotation::Quad & quad = ha->highlightQuads[ q ];
+                        const Okular::HighlightAnnotation::Quad & quad = ha->highlightQuads[ q ];
                         // normalize page point to image
                         for ( int i = 0; i < 4; i++ )
                         {
-                            NormalizedPoint point;
+                            Okular::NormalizedPoint point;
                             point.x = (quad.points[ i ].x - xOffset) * xScale;
                             point.y = (quad.points[ i ].y - yOffset) * yScale;
                             path.append( point );
@@ -341,29 +409,29 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
                         switch ( type )
                         {
                             // highlight the whole rect
-                            case HighlightAnnotation::Highlight:
+                            case Okular::HighlightAnnotation::Highlight:
                                 drawShapeOnImage( backImage, path, true, QPen(), a->style.color, pageScale, Multiply );
                                 break;
                             // highlight the bottom part of the rect
-                            case HighlightAnnotation::Squiggly:
-                                path[ 0 ].x = ( path[ 0 ].x + path[ 3 ].x ) / 2.0;
-                                path[ 0 ].y = ( path[ 0 ].y + path[ 3 ].y ) / 2.0;
-                                path[ 1 ].x = ( path[ 1 ].x + path[ 2 ].x ) / 2.0;
-                                path[ 1 ].y = ( path[ 1 ].y + path[ 2 ].y ) / 2.0;
+                            case Okular::HighlightAnnotation::Squiggly:
+                                path[ 3 ].x = ( path[ 0 ].x + path[ 3 ].x ) / 2.0;
+                                path[ 3 ].y = ( path[ 0 ].y + path[ 3 ].y ) / 2.0;
+                                path[ 2 ].x = ( path[ 1 ].x + path[ 2 ].x ) / 2.0;
+                                path[ 2 ].y = ( path[ 1 ].y + path[ 2 ].y ) / 2.0;
                                 drawShapeOnImage( backImage, path, true, QPen(), a->style.color, pageScale, Multiply );
                                 break;
                             // make a line at 3/4 of the height
-                            case HighlightAnnotation::Underline:
-                                path[ 0 ].x = ( path[ 0 ].x + 3*path[ 3 ].x ) / 4.0;
-                                path[ 0 ].y = ( path[ 0 ].y + 3*path[ 3 ].y ) / 4.0;
-                                path[ 1 ].x = ( path[ 1 ].x + 3*path[ 2 ].x ) / 4.0;
-                                path[ 1 ].y = ( path[ 1 ].y + 3*path[ 2 ].y ) / 4.0;
+                            case Okular::HighlightAnnotation::Underline:
+                                path[ 0 ].x = ( 3 * path[ 0 ].x + path[ 3 ].x ) / 4.0;
+                                path[ 0 ].y = ( 3 * path[ 0 ].y + path[ 3 ].y ) / 4.0;
+                                path[ 1 ].x = ( 3 * path[ 1 ].x + path[ 2 ].x ) / 4.0;
+                                path[ 1 ].y = ( 3 * path[ 1 ].y + path[ 2 ].y ) / 4.0;
                                 path.pop_back();
                                 path.pop_back();
                                 drawShapeOnImage( backImage, path, false, QPen( a->style.color, 2 ), QBrush(), pageScale );
                                 break;
                             // make a line at 1/2 of the height
-                            case HighlightAnnotation::StrikeOut:
+                            case Okular::HighlightAnnotation::StrikeOut:
                                 path[ 0 ].x = ( path[ 0 ].x + path[ 3 ].x ) / 2.0;
                                 path[ 0 ].y = ( path[ 0 ].y + path[ 3 ].y ) / 2.0;
                                 path[ 1 ].x = ( path[ 1 ].x + path[ 2 ].x ) / 2.0;
@@ -376,24 +444,24 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
                     }
                 }
                 // draw InkAnnotation MISSING:invar width, PENTRACER
-                else if ( type == Annotation::AInk )
+                else if ( type == Okular::Annotation::AInk )
                 {
                     // get the annotation
-                    InkAnnotation * ia = (InkAnnotation *) a;
+                    Okular::InkAnnotation * ia = (Okular::InkAnnotation *) a;
 
                     // draw each ink path
                     int paths = ia->inkPaths.size();
                     for ( int p = 0; p < paths; p++ )
                     {
                         NormalizedPath path;
-                        const QLinkedList<NormalizedPoint> & inkPath = ia->inkPaths[ p ];
+                        const QLinkedList<Okular::NormalizedPoint> & inkPath = ia->inkPaths[ p ];
 
                         // normalize page point to image
-                        QLinkedList<NormalizedPoint>::const_iterator pIt = inkPath.begin(), pEnd = inkPath.end();
+                        QLinkedList<Okular::NormalizedPoint>::const_iterator pIt = inkPath.begin(), pEnd = inkPath.end();
                         for ( ; pIt != pEnd; ++pIt )
                         {
-                            const NormalizedPoint & inkPoint = *pIt;
-                            NormalizedPoint point;
+                            const Okular::NormalizedPoint & inkPoint = *pIt;
+                            Okular::NormalizedPoint point;
                             point.x = (inkPoint.x - xOffset) * xScale;
                             point.y = (inkPoint.y - yOffset) * yScale;
                             path.append( point );
@@ -406,7 +474,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
         }
 
         // 4B.5. create the back pixmap converting from the local image
-        backPixmap = new QPixmap( backImage );
+        backPixmap = new QPixmap( QPixmap::fromImage( backImage ) );
 
         // 4B.6. create a painter over the pixmap and set it as the active one
         mixedPainter = new QPainter( backPixmap );
@@ -417,12 +485,12 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
     if ( unbufferedAnnotations )
     {
         // iterate over annotations and paint AText, AGeom, AStamp
-        QList< Annotation * >::const_iterator aIt = unbufferedAnnotations->begin(), aEnd = unbufferedAnnotations->end();
+        QList< Okular::Annotation * >::const_iterator aIt = unbufferedAnnotations->begin(), aEnd = unbufferedAnnotations->end();
         for ( ; aIt != aEnd; ++aIt )
         {
-            Annotation * a = *aIt;
+            Okular::Annotation * a = *aIt;
 
-            // honour opacity settings on supported types
+            // honor opacity settings on supported types
             unsigned int opacity = (unsigned int)( 255.0 * a->style.opacity );
             if ( opacity <= 0 )
                 continue;
@@ -433,56 +501,65 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
             QRect innerRect( annotRect.left() - annotBoundary.left(), annotRect.top() -
                     annotBoundary.top(), annotRect.width(), annotRect.height() );
 
-            Annotation::SubType type = a->subType();
+            Okular::Annotation::SubType type = a->subType();
 
-            // draw TextAnnotation (NOT only the 'Linked' variant)
-            if ( type == Annotation::AText )
+            // draw TextAnnotation
+            if ( type == Okular::Annotation::AText )
             {
-                TextAnnotation * text = (TextAnnotation *)a;
-                if ( text->textType == TextAnnotation::InPlace )
+                Okular::TextAnnotation * text = (Okular::TextAnnotation *)a;
+                if ( text->textType == Okular::TextAnnotation::InPlace )
                 {
-                    //Have the square of fixed size and adapting text font
-                    QFontMetricsF mf(text->textFont);
-                    QRectF rcf=mf.boundingRect(text->inplaceText);
-                    QSize sz(int(rcf.width()+5), int(rcf.height()+5));
-                    if(sz.height()<annotBoundary.height())
-                        sz.setHeight(annotBoundary.height());
-                    if(sz.width()<annotBoundary.width())
-                        sz.setWidth(annotBoundary.width());
-                    
-                    QPixmap pixmap(sz);
-                    pixmap.fill( a->style.color );
-                    QPainter painter;
-                    painter.begin( &pixmap );
-                    painter.setPen( Qt::black );//( NoPen );
-                    //painter.setBrush( bule);
-                    painter.drawRect( 0, 0, sz.width()-2, sz.height()-2 );
-                    //painter.drawText(2,sz.height()/2+int(rcf.height()/2),text->inplaceText);
-                    painter.drawText(0,0,sz.width(),sz.height(),
-                                     Qt::AlignVCenter|Qt::AlignLeft|Qt::TextWrapAnywhere,
-                                     text->inplaceText);
+                    QRect bigRect = a->boundary.geometry( (int)page->width(), (int)page->height() );
+
+                    // the strategy behind 'bigger': if where are we going to
+                    // draw is bigger than the Page, then draw the rect only
+                    // after then scaling, so it won't be wider than 1px;
+                    // otherwise draw it right after the text
+                    bool bigger = mixedPainter->device()->width() > page->width();
+                    QImage image( bigRect.size(), QImage::Format_ARGB32 );
+                    image.fill( qRgba( a->style.color.red(), a->style.color.green(), a->style.color.blue(), 255 ) );
+                    QPainter painter( &image );
+                    painter.setPen( Qt::black );
+                    painter.setFont( text->textFont );
+                    Qt::AlignmentFlag halign = ( text->inplaceAlign == 1 ? Qt::AlignHCenter : ( text->inplaceAlign == 2 ? Qt::AlignRight : Qt::AlignLeft ) );
+                    painter.drawText( 2, 2, image.width() - 2, image.height() - 2,
+                                      Qt::AlignTop | halign | Qt::TextWordWrap,
+                                      text->inplaceText );
+                    if ( !bigger )
+                        painter.drawRect( 0, 0, image.width() - 1, image.height() - 1 );
                     painter.end();
-                    QImage scaledImage;
-                    scalePixmapOnImage( scaledImage, &pixmap,
-                                        annotBoundary.width(),
-                                        annotBoundary.height(), innerRect );
-                    //colorizeImage( scaledImage, a->style.color, opacity );
-                    //scaledImage.setAlphaBuffer( true );
-                    pixmap = QPixmap::fromImage( scaledImage );
-                    
-                    mixedPainter->drawPixmap( annotRect.topLeft(), pixmap );
+                    image = image.scaled( annotBoundary.size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
+                    if ( bigger )
+                    {
+                        painter.begin( &image );
+                        painter.setPen( Qt::black );
+                        painter.drawRect( 0, 0, image.width() - 1, image.height() - 1 );
+                        painter.end();
+                    }
+
+//                    mixedPainter->drawImage( annotBoundary.topLeft(), image.scaled( annotBoundary.size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation ) );
+                    mixedPainter->drawImage( annotBoundary.topLeft(), image );
                 }
-                
-                if ( text->textType == TextAnnotation::Linked )
+                else if ( text->textType == Okular::TextAnnotation::Linked )
                 {
                 // get pixmap, colorize and alpha-blend it
-                    QPixmap pixmap = DesktopIcon( text->textIcon );
+                    QString path;
+                    QPixmap pixmap = KGlobal::iconLoader()->loadIcon( text->textIcon.toLower(), K3Icon::User, 32, K3Icon::DefaultState, &path, true );
+                    if ( path.isEmpty() )
+                        pixmap = KGlobal::iconLoader()->loadIcon( text->textIcon.toLower(), K3Icon::NoGroup, 32 );
                     QImage scaledImage;
+                    QRect annotBoundary2 = QRect( annotBoundary.topLeft(), QSize( TEXTANNOTATION_ICONSIZE, TEXTANNOTATION_ICONSIZE ) );
+                    QRect annotRect2 = annotBoundary2.intersect( limits );
+                    QRect innerRect2( annotRect2.left() - annotBoundary2.left(), annotRect2.top() -
+                    annotBoundary2.top(), annotRect2.width(), annotRect2.height() );
                     scalePixmapOnImage( scaledImage, &pixmap,
-                                        annotBoundary.width(),
-                                        annotBoundary.height(), innerRect );
-                    colorizeImage( scaledImage, a->style.color, opacity );
-                    scaledImage.setAlphaBuffer( true );
+                                        TEXTANNOTATION_ICONSIZE, TEXTANNOTATION_ICONSIZE,
+                                        innerRect2, QImage::Format_ARGB32 );
+                    // if the annotation color is valid (ie it was set), then
+                    // use it to colorize the icon, otherwise the icon will be
+                    // "gray"
+                    if ( a->style.color.isValid() )
+                        colorizeImage( scaledImage, a->style.color, opacity );
                     pixmap = QPixmap::fromImage( scaledImage );
 
                 // draw the mangled image to painter
@@ -491,18 +568,20 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
 
             }
             // draw StampAnnotation
-            else if ( type == Annotation::AStamp )
+            else if ( type == Okular::Annotation::AStamp )
             {
-                StampAnnotation * stamp = (StampAnnotation *)a;
+                Okular::StampAnnotation * stamp = (Okular::StampAnnotation *)a;
 
                 // get pixmap and alpha blend it if needed
-                QPixmap pixmap = DesktopIcon( stamp->stampIconName );
+                QString path;
+                QPixmap pixmap = KGlobal::iconLoader()->loadIcon( stamp->stampIconName.toLower(), K3Icon::User, qMin( annotBoundary.width(), annotBoundary.height() ), K3Icon::DefaultState, &path, true );
+                if ( path.isEmpty() )
+                    pixmap = KGlobal::iconLoader()->loadIcon( stamp->stampIconName.toLower(), K3Icon::NoGroup, qMin( annotBoundary.width(), annotBoundary.height() ) );
                 QImage scaledImage;
                 scalePixmapOnImage( scaledImage, &pixmap, annotBoundary.width(),
-                                    annotBoundary.height(), innerRect );
+                                    annotBoundary.height(), innerRect, QImage::Format_ARGB32 );
                 if ( opacity < 255 )
                     changeImageAlpha( scaledImage, opacity );
-                scaledImage.setAlphaBuffer( true );
                 pixmap = QPixmap::fromImage( scaledImage );
 
                 // draw the scaled and al
@@ -514,18 +593,17 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
                 //GeomAnnotation * geom = (GeomAnnotation *)a;
                 //if ( geom->geomType == GeomAnnotation::InscribedSquare )
                 //{
-                    QImage rectImage( innerRect.width(), innerRect.height(), 32 );
+                    QImage rectImage( innerRect.width(), innerRect.height(), QImage::Format_ARGB32 );
                     const QColor & c = a->style.color;
                     unsigned int color = qRgba( c.red(), c.green(), c.blue(), opacity );
                     rectImage.fill( color );
-                    rectImage.setAlphaBuffer( true );
                     mixedPainter->drawImage( annotRect.topLeft(), rectImage );
                 //}
                 //else if ( geom->geomType == GeomAnnotation::InscribedCircle )
             }
 
             // draw extents rectangle
-            if ( KpdfSettings::debugDrawAnnotationRect() )
+            if ( Okular::Settings::debugDrawAnnotationRect() )
             {
                 mixedPainter->setPen( a->style.color );
                 mixedPainter->drawRect( annotBoundary );
@@ -545,12 +623,12 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const KPDFPage * p
         QRect limitsEnlarged = limits;
         limitsEnlarged.adjust( -2, -2, 2, 2 );
         // draw rects that are inside the 'limits' paint region as opaque rects
-        QLinkedList< ObjectRect * >::const_iterator lIt = page->m_rects.begin(), lEnd = page->m_rects.end();
+        QLinkedList< Okular::ObjectRect * >::const_iterator lIt = page->m_rects.begin(), lEnd = page->m_rects.end();
         for ( ; lIt != lEnd; ++lIt )
         {
-            ObjectRect * rect = *lIt;
-            if ( (enhanceLinks && rect->objectType() == ObjectRect::Link) ||
-                 (enhanceImages && rect->objectType() == ObjectRect::Image) )
+            Okular::ObjectRect * rect = *lIt;
+            if ( (enhanceLinks && rect->objectType() == Okular::ObjectRect::Link) ||
+                 (enhanceImages && rect->objectType() == Okular::ObjectRect::Image) )
             {
                 if ( limitsEnlarged.intersects( rect->boundingRect( scaledWidth, scaledHeight ) ) )
                 {
@@ -588,13 +666,14 @@ void PagePainter::cropPixmapOnImage( QImage & dest, const QPixmap * src, const Q
     else
     {
         QPixmap croppedPixmap( r.width(), r.height() );
-        copyBlt( &croppedPixmap, 0, 0, src, r.left(), r.top(), r.width(), r.height() );
+        QPainter p( &croppedPixmap );
+        p.drawPixmap( 0, 0, *src, r.left(), r.top(), r.width(), r.height() );
         dest = croppedPixmap.toImage();
     }
 }
 
 void PagePainter::scalePixmapOnImage ( QImage & dest, const QPixmap * src,
-    int scaledWidth, int scaledHeight, const QRect & cropRect )
+    int scaledWidth, int scaledHeight, const QRect & cropRect, QImage::Format format )
 {
     // {source, destination, scaling} params
     int srcWidth = src->width(),
@@ -605,7 +684,7 @@ void PagePainter::scalePixmapOnImage ( QImage & dest, const QPixmap * src,
         destHeight = cropRect.height();
 
     // destination image (same geometry as the pageLimits rect)
-    dest = QImage( destWidth, destHeight, 32 );
+    dest = QImage( destWidth, destHeight, format );
     unsigned int * destData = (unsigned int *)dest.bits();
 
     // source image (1:1 conversion from pixmap)
@@ -613,7 +692,7 @@ void PagePainter::scalePixmapOnImage ( QImage & dest, const QPixmap * src,
     unsigned int * srcData = (unsigned int *)srcImage.bits();
 
     // precalc the x correspondancy conversion in a lookup table
-    unsigned int xOffset[ destWidth ];
+    QVarLengthArray<unsigned int> xOffset( destWidth );
     for ( int x = 0; x < destWidth; x++ )
         xOffset[ x ] = ((x + destLeft) * srcWidth) / scaledWidth;
 
@@ -694,7 +773,7 @@ void PagePainter::colorizeImage( QImage & grayImage, const QColor & color,
 // directory. This is to be replaced by Arthur calls for drawing antialiased
 // primitives, but until that AGG2 does its job very fast and good-looking.
 
-#include "kpdf_pixfmt_rgba.h"
+#include "okular_pixfmt_rgba.h"
 #include "agg_rendering_buffer.h"
 #include "agg_renderer_base.h"
 #include "agg_scanline_u.h"
