@@ -11,7 +11,10 @@
 #include <qfile.h>
 #include <qcolor.h>
 #include <qevent.h>
+#include <qlist.h>
 #include <qpainter.h>
+#include <qset.h>
+#include <qvariant.h>
 #include <kiconloader.h>
 #include <klocale.h>
 #include <kstandarddirs.h>
@@ -24,186 +27,16 @@
 #include <math.h>
 
 // local includes
+#include "core/area.h"
 #include "core/document.h"
 #include "core/page.h"
 #include "core/annotations.h"
 #include "settings.h"
+#include "annotationtools.h"
 #include "pageview.h"
 #include "pageviewutils.h"
 #include "pageviewannotator.h"
 
-/**
- * @short Engine: filter events to distill an Annotation.
- */
-class AnnotatorEngine
-{
-    public:
-        AnnotatorEngine( const QDomElement & engineElement )
-            : m_engineElement( engineElement ), m_creationCompleted( false )
-        {
-            // parse common engine attributes
-            if ( engineElement.hasAttribute( "color" ) )
-                m_engineColor = QColor( engineElement.attribute( "color" ) );
-        }
-        virtual ~AnnotatorEngine() {};
-
-        // enum definitions
-        enum EventType { Press, Move, Release };
-        enum Button { None, Left, Right };
-
-        // perform operations
-        virtual QRect event( EventType type, Button button, double nX, double nY, double xScale, double yScale ) = 0;
-        virtual void paint( QPainter * painter, double xScale, double yScale, const QRect & clipRect ) = 0;
-        virtual Annotation * end() = 0;
-
-        // query creation state
-        //PageViewItem * editingItem() const { return m_lockedItem; }
-        bool creationCompleted() const { return m_creationCompleted; }
-
-    protected:
-        // common engine attributes (the element includes annotation desc)
-        QDomElement m_engineElement;
-        QColor m_engineColor;
-        // other vars (remove this!)
-        bool m_creationCompleted;
-};
-
-/** @short SmoothPathEngine */
-class SmoothPathEngine : public AnnotatorEngine
-{
-    public:
-        SmoothPathEngine( const QDomElement & engineElement )
-            : AnnotatorEngine( engineElement )
-        {
-            // parse engine specific attributes
-        }
-
-        QRect event( EventType type, Button button, double nX, double nY, double xScale, double yScale )
-        {
-            // only proceed if pressing left button
-            if ( button != Left )
-                return QRect();
-
-            // start operation
-            if ( type == Press && points.isEmpty() )
-            {
-                lastPoint.x = nX;
-                lastPoint.y = nY;
-                totalRect.left = totalRect.right = lastPoint.x;
-                totalRect.top = totalRect.bottom = lastPoint.y;
-                points.append( lastPoint );
-            }
-            // add a point to the path
-            else if ( type == Move && points.count() > 0 )
-            {
-                //double dist = hypot( nX - points.last().x, nY - points.last().y );
-                //if ( dist > 0.0001 )
-                //{
-                    // append mouse position (as normalized point) to the list
-                    NormalizedPoint nextPoint = NormalizedPoint( nX, nY );
-                    points.append( nextPoint );
-                    // update total rect
-                    double dX = 2.0 / (double)xScale;
-                    double dY = 2.0 / (double)yScale;
-                    totalRect.left = qMin( totalRect.left, nX - dX );
-                    totalRect.top = qMin( totalRect.top, nY - dY );
-                    totalRect.right = qMax( nX + dX, totalRect.right );
-                    totalRect.bottom = qMax( nY + dY, totalRect.bottom );
-                    // paint the difference to previous full rect
-                    NormalizedRect incrementalRect;
-                    incrementalRect.left = qMin( nextPoint.x, lastPoint.x ) - dX;
-                    incrementalRect.right = qMax( nextPoint.x, lastPoint.x ) + dX;
-                    incrementalRect.top = qMin( nextPoint.y, lastPoint.y ) - dY;
-                    incrementalRect.bottom = qMax( nextPoint.y, lastPoint.y ) + dY;
-                    lastPoint = nextPoint;
-                    return incrementalRect.geometry( (int)xScale, (int)yScale );
-                //}
-            }
-            // terminate process
-            else if ( type == Release && points.count() > 0 )
-            {
-                if ( points.count() < 2 )
-                    points.clear();
-                else
-                    m_creationCompleted = true;
-                return totalRect.geometry( (int)xScale, (int)yScale );
-            }
-            return QRect();
-        }
-
-        void paint( QPainter * painter, double xScale, double yScale, const QRect & /*clipRect*/ )
-        {
-            // draw SmoothPaths with at least 2 points
-            if ( points.count() > 1 )
-            {
-                // use engine's color for painting
-                painter->setPen( QPen( m_engineColor, 1 ) );
-
-                QLinkedList<NormalizedPoint>::iterator pIt = points.begin(), pEnd = points.end();
-                NormalizedPoint pA = *pIt;
-                ++pIt;
-                for ( ; pIt != pEnd; ++pIt )
-                {
-                    NormalizedPoint pB = *pIt;
-                    painter->drawLine( (int)(pA.x * (double)xScale), (int)(pA.y * (double)yScale),
-                                    (int)(pB.x * (double)xScale), (int)(pB.y * (double)yScale) );
-                    pA = pB;
-                }
-            }
-        }
-
-        Annotation * end()
-        {
-            // find out annotation's description node
-            const QDomElement & annElement = m_engineElement.firstChild().toElement();
-            if ( annElement.isNull() || annElement.tagName() != "annotation" )
-                return 0;
-
-            // find out annotation's type
-            Annotation * ann = 0;
-            QString typeString = annElement.attribute( "type" );
-
-            // create HighlightAnnotation from path
-            if ( typeString == "Highlight" )
-            {
-                HighlightAnnotation * ha = new HighlightAnnotation();
-                ann = ha;
-                // TODO
-            }
-            // create InkAnnotation from path
-            else if ( typeString == "Ink" )
-            {
-                InkAnnotation * ia = new InkAnnotation();
-                ann = ia;
-                if ( annElement.hasAttribute( "width" ) )
-                    ann->style.width = annElement.attribute( "width" ).toDouble();
-                // fill points
-                ia->inkPaths.append( points );
-                // set boundaries
-                ia->boundary = totalRect;
-            }
-
-            // safety check
-            if ( !ann )
-                return 0;
-
-            // set common attributes
-            ann->style.color = annElement.hasAttribute( "color" ) ?
-                annElement.attribute( "color" ) : m_engineColor;
-            if ( annElement.hasAttribute( "opacity" ) )
-                ann->style.opacity = annElement.attribute( "opacity" ).toDouble();
-
-            // return annotation
-            return ann;
-        }
-
-    private:
-        // data
-        QLinkedList<NormalizedPoint> points;
-        NormalizedRect totalRect;
-        NormalizedPoint lastPoint;
-};
-
 /** @short PickPointEngine */
 class PickPointEngine : public AnnotatorEngine
 {
@@ -212,12 +45,20 @@ class PickPointEngine : public AnnotatorEngine
     : AnnotatorEngine( engineElement ), clicked( false ), xscale(1.0), yscale(1.0)
         {
             // parse engine specific attributes
-            QString pixmapName = engineElement.attribute( "hoverIcon" );
-            if ( pixmapName.isNull() )
+            pixmapName = engineElement.attribute( "hoverIcon" );
+            QString stampname = m_annotElement.attribute( "icon" );
+            if ( m_annotElement.attribute( "type" ) == "Stamp" && !stampname.simplified().isEmpty() )
+                pixmapName = stampname;
+            if ( pixmapName.simplified().isEmpty() )
                 pixmapName = "okular";
+            center = QVariant( engineElement.attribute( "center" ) ).toBool();
+            bool ok = true;
+            size = engineElement.attribute( "size", "32" ).toInt( &ok );
+            if ( !ok )
+                size = 32;
 
             // create engine objects
-            pixmap = new QPixmap( DesktopIcon( "okular", 32 ) );
+            pixmap = new QPixmap( DesktopIcon( pixmapName, size ) );
         }
 
         ~PickPointEngine()
@@ -225,10 +66,12 @@ class PickPointEngine : public AnnotatorEngine
             delete pixmap;
         }
 
-        QRect event( EventType type, Button button, double nX, double nY, double xScale, double yScale )
+        QRect event( EventType type, Button button, double nX, double nY, double xScale, double yScale, const Okular::Page * page )
         {
             xscale=xScale;
             yscale=yScale;
+            pagewidth = page->width();
+            pageheight = page->height();
             // only proceed if pressing left button
             if ( button != Left )
                 return QRect();
@@ -255,72 +98,77 @@ class PickPointEngine : public AnnotatorEngine
             // update variables and extents (zoom invariant rect)
             point.x = nX;
             point.y = nY;
-            rect.left = nX - (16.0 / (double)xScale) ;
-            rect.right = nX + (17.0 / (double)xScale) ;
-            rect.top = nY - (16.0 / (double)yScale) ;
-            rect.bottom = nY + (17.0 / (double)yScale) ;
-            return rect.geometry( (int)xScale, (int)yScale );
+            if ( center )
+            {
+                rect.left = nX - ( pixmap->width() / ( xScale * 2.0 ) );
+                rect.top = nY - ( pixmap->height() / ( yScale * 2.0 ) );
+            }
+            else
+            {
+                rect.left = nX;
+                rect.top = nY;
+            }
+            rect.right = rect.left + ( pixmap->width() / xScale );
+            rect.bottom = rect.top + ( pixmap->height() / yScale );
+            return rect.geometry( (int)xScale, (int)yScale ).adjusted( 0, 0, 1, 1 );
         }
 
         void paint( QPainter * painter, double xScale, double yScale, const QRect & /*clipRect*/ )
         {
             if ( clicked && pixmap )
             {
-                int pX = (int)(point.x * (double)xScale);
-                int pY = (int)(point.y * (double)yScale);
-                painter->drawPixmap( pX - 15, pY - 15, *pixmap );
+                painter->drawPixmap( QPointF( rect.left * xScale, rect.top * yScale ), *pixmap );
             }
         }
 
-        Annotation * end()
+        QList< Okular::Annotation* > end()
         {
+            m_creationCompleted = false;
+
             // find out annotation's description node
-            const QDomElement & annElement = m_engineElement.firstChild().toElement();
-            if ( annElement.isNull() || annElement.tagName() != "annotation" )
-                return 0;
+            if ( m_annotElement.isNull() )
+                return QList< Okular::Annotation* >();
             
             // find out annotation's type
-            Annotation * ann = 0;
-            QString typeString = annElement.attribute( "type" );
+            Okular::Annotation * ann = 0;
+            QString typeString = m_annotElement.attribute( "type" );
             // create TextAnnotation from path
             if ( typeString == "FreeText")	//<annotation type="Text"
             {
                 //note dialog
                 QString prompt = i18n( "Please input the free text:" ) ;
                 bool resok;
-                QString note ="";
-
-                note= KInputDialog::getText( i18n("FreeText"), prompt, note,&resok );
+                QString note = KInputDialog::getMultiLineText( i18n( "Free Text" ), prompt, QString::null, &resok );
                 if(resok)
                 {
                     //add note
-                    TextAnnotation * ta = new TextAnnotation();
+                    Okular::TextAnnotation * ta = new Okular::TextAnnotation();
                     ann = ta;
                     ta->inplaceText=note;
-                    ta->textType = TextAnnotation::InPlace;
+                    ta->textType = Okular::TextAnnotation::InPlace;
                     //set boundary
-                    QFontMetricsF mf(ta->textFont);
-                    QRectF rcf=mf.boundingRect(ta->inplaceText);
                     rect.left = qMin(startpoint.x,point.x);
                     rect.top = qMin(startpoint.y,point.y);
                     rect.right = qMax(startpoint.x,point.x);
                     rect.bottom = qMax(startpoint.y,point.y);
                     kDebug()<<"astario:   xyScale="<<xscale<<","<<yscale<<endl;
-                    double pixfactor=0.002;
-                    rect.right = qMax(rect.right, rect.left+rcf.width()*pixfactor);
-                    rect.bottom = qMax(rect.bottom, rect.top+rcf.height()*pixfactor*xscale/yscale);
+                    static int padding = 2;
+                    QFontMetricsF mf(ta->textFont);
+                    QRectF rcf = mf.boundingRect( Okular::NormalizedRect( rect.left, rect.top, 1.0, 1.0 ).geometry( (int)pagewidth, (int)pageheight ).adjusted( padding, padding, -padding, -padding ),
+                                                  Qt::AlignTop | Qt::AlignLeft | Qt::TextWordWrap, ta->inplaceText );
+                    rect.right = qMax(rect.right, rect.left+(rcf.width()+padding*2)/pagewidth);
+                    rect.bottom = qMax(rect.bottom, rect.top+(rcf.height()+padding*2)/pageheight);
                     ta->boundary=this->rect;
                     ta->window.summary="TextBox";
                 }
             }
             else if ( typeString == "Text")
             {
-                TextAnnotation * ta = new TextAnnotation();
+                Okular::TextAnnotation * ta = new Okular::TextAnnotation();
                 ann = ta;
-                ta->textType = TextAnnotation::Linked;
+                ta->textType = Okular::TextAnnotation::Linked;
                 ta->window.text="";
-                //ta->window.flags &= ~(Annotation::Hidden);
-                ta->textIcon="comment";
+                //ta->window.flags &= ~(Okular::Annotation::Hidden);
                 double iconhei=0.03;
                 rect.left = point.x;
                 rect.top = point.y;
@@ -332,94 +180,134 @@ class PickPointEngine : public AnnotatorEngine
             // create StampAnnotation from path
             else if ( typeString == "Stamp" )
             {
-                StampAnnotation * sa = new StampAnnotation();
+                Okular::StampAnnotation * sa = new Okular::StampAnnotation();
                 ann = sa;
-                sa->stampIconName="okular";
+                sa->stampIconName = pixmapName;
+                double stampxscale = size / xscale;
+                double stampyscale = size / yscale;
+                if ( center )
+                {
+                    rect.left = point.x - stampxscale / 2;
+                    rect.top = point.y - stampyscale / 2;
+                }
+                else
+                {
+                    rect.left = point.x;
+                    rect.top = point.y;
+                }
+                rect.right = rect.left + stampxscale;
+                rect.bottom = rect.top + stampyscale;
+                sa->boundary = rect;
             }
 
             // safety check
             if ( !ann )
-                return 0;
+                return QList< Okular::Annotation* >();
 
             // set common attributes
-            ann->style.color = annElement.hasAttribute( "color" ) ?
-                    annElement.attribute( "color" ) : m_engineColor;
-            if ( annElement.hasAttribute( "opacity" ) )
-                ann->style.opacity = annElement.attribute( "opacity" ).toDouble();
-            ann->creationDate=ann->modifyDate=QDateTime::currentDateTime();
+            ann->style.color = m_annotElement.hasAttribute( "color" ) ?
+                    m_annotElement.attribute( "color" ) : m_engineColor;
+            if ( m_annotElement.hasAttribute( "opacity" ) )
+                ann->style.opacity = m_annotElement.attribute( "opacity", "1.0" ).toDouble();
 
             // return annotation
-            return ann;
+            return QList< Okular::Annotation* >() << ann;
         }
 
     private:
         bool clicked;
-        NormalizedRect rect;
-        NormalizedPoint startpoint;
-        NormalizedPoint point;
+        Okular::NormalizedRect rect;
+        Okular::NormalizedPoint startpoint;
+        Okular::NormalizedPoint point;
         QPixmap * pixmap;
+        QString pixmapName;
+        int size;
         double xscale,yscale;
+        double pagewidth, pageheight;
+        bool center;
 };
 
-/** @short TwoPointsEngine */
-class TwoPointsEngine : public AnnotatorEngine
+/** @short PolyLineEngine */
+class PolyLineEngine : public AnnotatorEngine
 {
     public:
-        TwoPointsEngine( const QDomElement & engineElement )
-            : AnnotatorEngine( engineElement )
+        PolyLineEngine( const QDomElement & engineElement )
+            : AnnotatorEngine( engineElement ), last( false )
         {
             // parse engine specific attributes
             m_block = engineElement.attribute( "block" ) == "true";
+            bool ok = true;
+            // numofpoints represents the max number of points for the current
+            // polygon/polyline, with a pair of exceptions:
+            // -1 means: the polyline must close on the first point (polygon)
+            // 0 means: construct as many points as you want, right-click
+            //   to construct the last point
+            numofpoints = engineElement.attribute( "points" ).toInt( &ok );
+            if ( !ok )
+                numofpoints = -1;
         }
 
-        QRect event( EventType type, Button button, double nX, double nY, double xScale, double yScale )
+        QRect event( EventType type, Button button, double nX, double nY, double xScale, double yScale, const Okular::Page * /*page*/ )
         {
             // only proceed if pressing left button
-            if ( button != Left )
-                return QRect();
+//            if ( button != Left )
+//                return rect;
 
             // start operation
-            if ( type == Press && points.isEmpty() )
+            if ( type == Press )
             {
-                NormalizedPoint newPoint;
                 newPoint.x = nX;
                 newPoint.y = nY;
-                rect.left = rect.right =newPoint.x;
-                rect.top = rect.bottom =newPoint.y;
-                points.append( newPoint );
-                return QRect();
+                if ( button == Right )
+                    last = true;
             }
             // move the second point
-            else if ( type == Move && !points.isEmpty() )
+            else if ( type == Move )
             {
-                if ( points.count() == 2 )
-                    points.pop_back();
-                NormalizedPoint newPoint;
-                newPoint.x = nX;
-                newPoint.y = nY;
-                points.append( newPoint );
-                NormalizedPoint firstPoint = points.front();
-                rect.left = qMin( firstPoint.x, nX ) - 2.0 / (double)xScale;
-                rect.right = qMax( firstPoint.x, nX ) + 2.0 / (double)xScale;
-                rect.top = qMin( firstPoint.y, nY ) - 2.0 / (double)yScale;
-                rect.bottom = qMax( firstPoint.y, nY ) + 2.0 / (double)yScale;
+                movingpoint.x = nX;
+                movingpoint.y = nY;
+                QRect oldmovingrect = movingrect;
+                movingrect = rect | QRect( (int)( movingpoint.x * xScale ), (int)( movingpoint.y * yScale ), 1, 1 );
+                return oldmovingrect | movingrect;
             }
-            // end creation if we have 2 points
-            else if ( type == Release && points.count() == 2 )
-                m_creationCompleted = true;
+            else if ( type == Release )
+            {
+                Okular::NormalizedPoint tmppoint;
+                tmppoint.x = nX;
+                tmppoint.y = nY;
+                if ( fabs( tmppoint.x - newPoint.x + tmppoint.y - newPoint.y ) > 1e-2 )
+                    return rect;
 
-            return rect.geometry( (int)xScale, (int)yScale );
+                if ( numofpoints == -1 && points.count() > 1 && ( fabs( points[0].x - newPoint.x + points[0].y - newPoint.y ) < 1e-2 ) )
+                {
+                    last = true;
+                }
+                else
+                {
+                    points.append( newPoint );
+                    rect |= QRect( (int)( newPoint.x * xScale ), (int)( newPoint.y * yScale ), 1, 1 );
+                }
+                // end creation if we have constructed the last point of enough points
+                if ( last || points.count() == numofpoints )
+                {
+                    m_creationCompleted = true;
+                    last = false;
+                    normRect = Okular::NormalizedRect( rect, xScale, yScale );
+                }
+            }
+
+            return rect;
         }
 
         void paint( QPainter * painter, double xScale, double yScale, const QRect & /*clipRect*/ )
         {
-            if ( points.count() != 2 )
+            if ( points.count() < 1 )
                 return;
 
-            NormalizedPoint first = points[0];
-            NormalizedPoint second = points[1];
-            if ( m_block )
+            if ( m_block && points.count() == 2 )
             {
+                Okular::NormalizedPoint first = points[0];
+                Okular::NormalizedPoint second = points[1];
                 // draw a semitransparent block around the 2 points
                 painter->setPen( m_engineColor );
                 painter->setBrush( QBrush( m_engineColor.light(), Qt::Dense4Pattern ) );
@@ -428,62 +316,223 @@ class TwoPointsEngine : public AnnotatorEngine
             }
             else
             {
-                // draw a line that connects the 2 points
+                // draw a polyline that connects the constructed points
                 painter->setPen( QPen( m_engineColor, 2 ) );
-                painter->drawLine( (int)(first.x * (double)xScale), (int)(first.y * (double)yScale),
-                                   (int)(second.x * (double)xScale), (int)(second.y * (double)yScale) );
+                for ( int i = 1; i < points.count(); ++i )
+                    painter->drawLine( (int)(points[i - 1].x * (double)xScale), (int)(points[i - 1].y * (double)yScale),
+                                       (int)(points[i].x * (double)xScale), (int)(points[i].y * (double)yScale) );
+                painter->drawLine( (int)(points.last().x * (double)xScale), (int)(points.last().y * (double)yScale),
+                                   (int)(movingpoint.x * (double)xScale), (int)(movingpoint.y * (double)yScale) );
             }
         }
 
-        Annotation * end()
+        QList< Okular::Annotation* > end()
         {
+            m_creationCompleted = false;
+
 			// find out annotation's description node
-            const QDomElement & annElement = m_engineElement.firstChild().toElement();
-            if ( annElement.isNull() || annElement.tagName() != "annotation" )
-                return 0;
+            if ( m_annotElement.isNull() )
+                return QList< Okular::Annotation* >();
 
             // find out annotation's type
-            Annotation * ann = 0;
-            QString typeString = annElement.attribute( "type" );
+            Okular::Annotation * ann = 0;
+            QString typeString = m_annotElement.attribute( "type" );
 
             // create LineAnnotation from path
-            if ( typeString == "Line")	//<annotation type="Text"
+            if ( typeString == "Line" || typeString == "Polyline" || typeString == "Polygon" )
             {
-                if ( points.count() != 2 )
-                    return 0;
+                if ( points.count() < 2 )
+                    return QList< Okular::Annotation* >();
                 //add note
-                LineAnnotation * la = new LineAnnotation();
+                Okular::LineAnnotation * la = new Okular::LineAnnotation();
                 ann = la;
-                la->linePoints.append(points[0]);
-                la->linePoints.append(points[1]);
-                la->boundary=this->rect;
+                for ( int i = 0; i < points.count(); ++i )
+                    la->linePoints.append( points[i] );
+                if ( numofpoints == -1 )
+                    la->lineClosed = true;
+                la->boundary = normRect;
 
             }
 
             // safety check
             if ( !ann )
-                return 0;
+                return QList< Okular::Annotation* >();
 
             // set common attributes
-            ann->style.color = annElement.hasAttribute( "color" ) ?
-                    annElement.attribute( "color" ) : m_engineColor;
-            if ( annElement.hasAttribute( "opacity" ) )
-                ann->style.opacity = annElement.attribute( "opacity" ).toDouble();
+            ann->style.color = m_annotElement.hasAttribute( "color" ) ?
+                    m_annotElement.attribute( "color" ) : m_engineColor;
+            if ( m_annotElement.hasAttribute( "opacity" ) )
+                ann->style.opacity = m_annotElement.attribute( "opacity", "1.0" ).toDouble();
             // return annotation
 
-            return ann;
+            return QList< Okular::Annotation* >() << ann;
         }
 
     private:
-        QList<NormalizedPoint> points;
-        NormalizedRect rect;
+        QList<Okular::NormalizedPoint> points;
+        Okular::NormalizedPoint newPoint;
+        Okular::NormalizedPoint movingpoint;
+        QRect rect;
+        QRect movingrect;
+        Okular::NormalizedRect normRect;
         bool m_block;
+        bool last;
+        int numofpoints;
+};
+
+/** @short TextSelectorEngine */
+class TextSelectorEngine : public AnnotatorEngine
+{
+    public:
+        TextSelectorEngine( const QDomElement & engineElement, PageView * pageView )
+            : AnnotatorEngine( engineElement ), m_pageView( pageView ),
+            selection( 0 )
+        {
+            // parse engine specific attributes
+        }
+
+        QRect event( EventType type, Button button, double nX, double nY, double xScale, double yScale, const Okular::Page * /*page*/ )
+        {
+            // only proceed if pressing left button
+            if ( button != Left )
+                return QRect();
+
+            if ( type == Press )
+            {
+                lastPoint.x = nX;
+                lastPoint.y = nY;
+                QRect oldrect = rect;
+                rect = QRect();
+                return oldrect;
+            }
+            else if ( type == Move )
+            {
+                if ( item() )
+                {
+                    QPoint start( (int)( lastPoint.x * item()->width() ), (int)( lastPoint.y * item()->height() ) );
+                    QPoint end( (int)( nX * item()->width() ), (int)( nY * item()->height() ) );
+                    Okular::RegularAreaRect * newselection = m_pageView->textSelectionForItem( item(), start, end );
+                    QList<QRect> * geom = newselection->geometry( (int)xScale, (int)yScale );
+                    QRect newrect;
+                    if ( geom )
+                    {
+                        foreach( QRect r, *geom )
+                        {
+                            if ( newrect.isNull() )
+                                newrect = r;
+                            else
+                                newrect |= r;
+                        }
+                    }
+                    rect |= newrect;
+                    delete selection;
+                    selection = newselection;
+                }
+            }
+            else if ( type == Release )
+            {
+                m_creationCompleted = true;
+            }
+            return rect;
+        }
+
+        void paint( QPainter * painter, double xScale, double yScale, const QRect & /*clipRect*/ )
+        {
+            if ( selection )
+            {
+                painter->setPen( Qt::NoPen );
+                QColor col = m_engineColor;
+                col.setAlphaF( 0.5 );
+                painter->setBrush( col );
+                foreach( Okular::NormalizedRect * r, *selection )
+                {
+                    painter->drawRect( r->geometry( (int)xScale, (int)yScale ) );
+                }
+            }
+        }
+
+        QList< Okular::Annotation* > end()
+        {
+            m_creationCompleted = false;
+
+            // find out annotation's description node
+            if ( m_annotElement.isNull() )
+                return QList< Okular::Annotation* >();
+
+            // find out annotation's type
+            Okular::Annotation * ann = 0;
+            QString typeString = m_annotElement.attribute( "type" );
+
+            Okular::HighlightAnnotation::HighlightType type = Okular::HighlightAnnotation::Highlight;
+            bool typevalid = false;
+            // create HighlightAnnotation's from the selected area
+            if ( typeString == "Highlight" )
+            {
+                type = Okular::HighlightAnnotation::Highlight;
+                typevalid = true;
+            }
+            else if ( typeString == "Squiggly" )
+            {
+                type = Okular::HighlightAnnotation::Squiggly;
+                typevalid = true;
+            }
+            else if ( typeString == "Underline" )
+            {
+                type = Okular::HighlightAnnotation::Underline;
+                typevalid = true;
+            }
+            else if ( typeString == "StrikeOut" )
+            {
+                type = Okular::HighlightAnnotation::StrikeOut;
+                typevalid = true;
+            }
+            if ( typevalid )
+            {
+                Okular::HighlightAnnotation * ha = new Okular::HighlightAnnotation();
+                ha->highlightType = type;
+                ha->boundary = Okular::NormalizedRect( rect, (int)item()->width(), (int)item()->height() );
+                foreach ( Okular::NormalizedRect * rect, *selection )
+                {
+                    Okular::HighlightAnnotation::Quad q;
+                    q.capStart = false;
+                    q.capEnd = false;
+                    q.feather = 1.0;
+                    q.points[0] = Okular::NormalizedPoint( rect->left, rect->bottom );
+                    q.points[1] = Okular::NormalizedPoint( rect->right, rect->bottom );
+                    q.points[2] = Okular::NormalizedPoint( rect->right, rect->top );
+                    q.points[3] = Okular::NormalizedPoint( rect->left, rect->top );
+                    ha->highlightQuads.append( q );
+                }
+                ann = ha;
+            }
+
+            // safety check
+            if ( !ann )
+                return QList< Okular::Annotation* >();
+
+            // set common attributes
+            ann->style.color = m_annotElement.hasAttribute( "color" ) ?
+                m_annotElement.attribute( "color" ) : m_engineColor;
+            if ( m_annotElement.hasAttribute( "opacity" ) )
+                ann->style.opacity = m_annotElement.attribute( "opacity", "1.0" ).toDouble();
+
+            // return annotations
+            return QList< Okular::Annotation* >() << ann;
+        }
+
+    private:
+        // data
+        PageView * m_pageView;
+        // TODO: support more pages
+        Okular::RegularAreaRect * selection;
+        Okular::NormalizedPoint lastPoint;
+        QRect rect;
 };
 
 
 /** PageViewAnnotator **/
 
-PageViewAnnotator::PageViewAnnotator( PageView * parent, KPDFDocument * storage )
+PageViewAnnotator::PageViewAnnotator( PageView * parent, Okular::Document * storage )
     : QObject( parent ), m_document( storage ), m_pageView( parent ),
     m_toolBar( 0 ), m_engine( 0 ), m_lastToolID( -1 ), m_lockedItem( 0 )
 {
@@ -553,10 +602,10 @@ void PageViewAnnotator::setEnabled( bool on )
     }
 
     // show the toolBar
-    m_toolBar->showItems( (PageViewToolBar::Side)KpdfSettings::editToolBarPlacement(), items );
+    m_toolBar->showItems( (PageViewToolBar::Side)Okular::Settings::editToolBarPlacement(), items );
 
     // ask for Author's name if not already set
-    if ( KpdfSettings::annotationsAuthor().isEmpty() )
+    if ( Okular::Settings::annotationsAuthor().isEmpty() )
     {
         // get default username from the kdelibs/kdecore/KUser
         KUser currentUser;
@@ -571,8 +620,8 @@ void PageViewAnnotator::setEnabled( bool on )
             firstTry = false;
         }
         // save the name
-        KpdfSettings::setAnnotationsAuthor( userName );
-        KpdfSettings::writeConfig();
+        Okular::Settings::setAnnotationsAuthor( userName );
+        Okular::Settings::writeConfig();
     }
 }
 
@@ -581,9 +630,9 @@ bool PageViewAnnotator::routeEvents() const
     return m_engine && m_toolBar;
 }
 
-void PageViewAnnotator::routeEvent( QMouseEvent * e, PageViewItem * item )
+QRect PageViewAnnotator::routeEvent( QMouseEvent * e, PageViewItem * item )
 {
-if ( !item ) return; //STRAPAAAATCH !!! FIXME
+    if ( !item ) return QRect();
 
     // find out mouse event type
     AnnotatorEngine::EventType eventType = AnnotatorEngine::Press;
@@ -594,7 +643,7 @@ if ( !item ) return; //STRAPAAAATCH !!! FIXME
 
     // find out the pressed button
     AnnotatorEngine::Button button = AnnotatorEngine::None;
-    Qt::ButtonState buttonState = ( eventType == AnnotatorEngine::Move ) ? e->state() : e->button();
+    Qt::MouseButtons buttonState = ( eventType == AnnotatorEngine::Move ) ? e->buttons() : e->button();
     if ( buttonState == Qt::LeftButton )
         button = AnnotatorEngine::Left;
     else if ( buttonState == Qt::RightButton )
@@ -607,14 +656,19 @@ if ( !item ) return; //STRAPAAAATCH !!! FIXME
     double nX = (double)(e->x() - itemRect.left()) / itemWidth;
     double nY = (double)(e->y() - itemRect.top()) / itemHeight;
 
+    QRect modifiedRect;
+
     // 1. lock engine to current item
     if ( m_lockedItem && item != m_lockedItem )
-        return;
+        return QRect();
     if ( !m_lockedItem && eventType == AnnotatorEngine::Press )
+    {
         m_lockedItem = item;
+        m_engine->setItem( m_lockedItem );
+    }
 
     // 2. use engine to perform operations
-    QRect paintRect = m_engine->event( eventType, button, nX, nY, itemWidth, itemHeight );
+    QRect paintRect = m_engine->event( eventType, button, nX, nY, itemWidth, itemHeight, item->page() );
 
     // 3. update absolute extents rect and send paint event(s)
     if ( paintRect.isValid() )
@@ -626,36 +680,46 @@ if ( !item ) return; //STRAPAAAATCH !!! FIXME
         // 3.2. decompose paint region in rects and send paint events
         QVector<QRect> rects = compoundRegion.unite( m_lastDrawnRect ).rects();
         for ( int i = 0; i < rects.count(); i++ )
-            m_pageView->updateContents( rects[i] );
+            m_pageView->widget()->update( rects[i] );
+        modifiedRect = compoundRegion.boundingRect() | m_lastDrawnRect;
     }
 
     // 4. if engine has finished, apply Annotation to the page
     if ( m_engine->creationCompleted() )
     {
-        // apply engine data to Annotation and reset engine
-        Annotation * annotation = m_engine->end();
-        // attach the newly filled annotation to the page
-        if ( annotation )
+        // apply engine data to the Annotation's and reset engine
+        QList< Okular::Annotation* > annotations = m_engine->end();
+        // attach the newly filled annotations to the page
+        foreach ( Okular::Annotation * annotation, annotations )
         {
-            annotation->author = KpdfSettings::annotationsAuthor();
+            if ( !annotation ) continue;
+
+            annotation->creationDate = annotation->modifyDate = QDateTime::currentDateTime();
+            annotation->author = Okular::Settings::annotationsAuthor();
             m_document->addPageAnnotation( m_lockedItem->pageNumber(), annotation );
         }
 
         // go on creating annotations of the same type
         slotToolSelected( m_lastToolID );
     }
+
+    return modifiedRect;
 }
 
 bool PageViewAnnotator::routePaints( const QRect & wantedRect ) const
 {
-    return m_engine && m_toolBar && wantedRect.intersects( m_lastDrawnRect );
+    return m_engine && m_toolBar && wantedRect.intersects( m_lastDrawnRect ) && m_lockedItem;
 }
 
 void PageViewAnnotator::routePaint( QPainter * painter, const QRect & paintRect )
 {
+    // if there's no locked item, then there's no decided place to draw on
+    if ( !m_lockedItem )
+        return;
+
 #ifndef NDEBUG
     // [DEBUG] draw the paint region if enabled
-    if ( KpdfSettings::debugDrawAnnotationRect() )
+    if ( Okular::Settings::debugDrawAnnotationRect() )
         painter->drawRect( paintRect );
 #endif
     // move painter to current itemGeometry rect
@@ -683,7 +747,7 @@ void PageViewAnnotator::slotToolSelected( int toolID )
     m_lockedItem = 0;
     if ( m_lastDrawnRect.isValid() )
     {
-        m_pageView->updateContents( m_lastDrawnRect );
+        m_pageView->widget()->update( m_lastDrawnRect );
         m_lastDrawnRect = QRect();
     }
 
@@ -723,14 +787,16 @@ void PageViewAnnotator::slotToolSelected( int toolID )
                     m_engine = new SmoothPathEngine( toolSubElement );
                 else if ( type == "PickPoint" )
                     m_engine = new PickPointEngine( toolSubElement );
-                else if ( type == "TwoPoints" )
-                    m_engine = new TwoPointsEngine( toolSubElement );
+                else if ( type == "PolyLine" )
+                    m_engine = new PolyLineEngine( toolSubElement );
+                else if ( type == "TextSelector" )
+                    m_engine = new TextSelectorEngine( toolSubElement, m_pageView );
                 else
                     kWarning() << "tools.xml: engine type:'" << type << "' is not defined!" << endl;
             }
             // display the tooltip
             else if ( toolSubElement.tagName() == "tooltip" )
-                m_pageView->displayMessage( toolSubElement.text() );
+                m_pageView->displayMessage( toolSubElement.text(), PageViewMessage::Annotation );
         }
 
         // consistancy warning
@@ -744,22 +810,8 @@ void PageViewAnnotator::slotToolSelected( int toolID )
 
 void PageViewAnnotator::slotSaveToolbarOrientation( int side )
 {
-    KpdfSettings::setEditToolBarPlacement( (int)side );
-    KpdfSettings::writeConfig();
-}
-
-Annotation* PageViewAnnotator::getAnnotationbyPos(const KPDFPage * page, double nX, double nY )
-{
-    QLinkedList< Annotation * >::const_iterator aIt = page->m_annotations.begin(), aEnd =page->m_annotations.end();
-    for ( ; aIt != aEnd; ++aIt )
-    {
-        Annotation * ann = *aIt;
-        if ( ann->boundary.contains( nX, nY ) )
-        {
-            return ann;
-        }
-    }
-    return 0;
+    Okular::Settings::setEditToolBarPlacement( (int)side );
+    Okular::Settings::writeConfig();
 }
 
 #include "pageviewannotator.moc"
