@@ -11,11 +11,13 @@
 #include <qapplication.h>
 #include <qbitmap.h>
 #include <qimage.h>
+#include <qlabel.h>
+#include <qlayout.h>
 #include <qpainter.h>
 #include <qevent.h>
+#include <qstyle.h>
 #include <qtimer.h>
-#include <qpushbutton.h>
-#include <QStyleOptionButton>
+#include <qtoolbutton.h>
 #include <kacceleratormanager.h>
 #include <kiconloader.h>
 #include <kimageeffect.h>
@@ -33,12 +35,12 @@
 /** PageViewItem     */
 /*********************/
 
-PageViewItem::PageViewItem( const KPDFPage * page )
+PageViewItem::PageViewItem( const Okular::Page * page )
     : m_page( page ), m_zoomFactor( 1.0 )
 {
 }
 
-const KPDFPage * PageViewItem::page() const
+const Okular::Page * PageViewItem::page() const
 {
     return m_page;
 }
@@ -104,7 +106,9 @@ PageViewMessage::PageViewMessage( QWidget * parent )
     QPalette pal = palette();
     pal.setColor( QPalette::Active, QPalette::Window, QApplication::palette().color( QPalette::Active, QPalette::Window ) );
     setPalette( pal );
-    move( 10, 10 );
+    // if the layout is LtR, we can safely place it in the right position
+    if ( QApplication::isLeftToRight() )
+        move( 10, 10 );
     resize( 0, 0 );
     hide();
 }
@@ -113,7 +117,7 @@ void PageViewMessage::display( const QString & message, Icon icon, int durationM
 // give Caesar what belongs to Caesar: code taken from Amarok's osd.h/.cpp
 // "redde (reddite, pl.) cesari quae sunt cesaris", just btw.  :)
 {
-    if ( !KpdfSettings::showOSD() )
+    if ( !Okular::Settings::showOSD() )
     {
         hide();
         return;
@@ -187,6 +191,11 @@ void PageViewMessage::display( const QString & message, Icon icon, int durationM
     bufferPainter.setPen( pal.color( QPalette::WindowText ) );
     bufferPainter.drawText( 5 + textXOffset, yText, message );
 
+    // if the layout is RtL, we can move it to the right place only after we
+    // know how much size it will take
+    if ( QApplication::isRightToLeft() )
+        move( parentWidget()->width() - geometry.width() - 10 - 1, 10 );
+
     // show widget and schedule a repaint
     show();
     update();
@@ -208,7 +217,8 @@ void PageViewMessage::display( const QString & message, Icon icon, int durationM
 void PageViewMessage::paintEvent( QPaintEvent * e )
 {
     // paint the internal pixmap over the widget
-    bitBlt( this, e->rect().topLeft(), &m_pixmap, e->rect() );
+    QPainter p( this );
+    p.drawPixmap( e->rect().topLeft(), m_pixmap, e->rect() );
 }
 
 void PageViewMessage::mousePressEvent( QMouseEvent * /*e*/ )
@@ -219,38 +229,68 @@ void PageViewMessage::mousePressEvent( QMouseEvent * /*e*/ )
 }
 
 
+/************************/
+/** PageViewTopMessage  */
+/************************/
+
+PageViewTopMessage::PageViewTopMessage( QWidget * parent )
+    : QWidget( parent )
+{
+    setAutoFillBackground( true );
+    QPalette pal = palette();
+    pal.setColor( QPalette::Active, QPalette::Window, Qt::yellow );
+    pal.setColor( QPalette::Inactive, QPalette::Window, Qt::yellow );
+    pal.setColor( QPalette::Disabled, QPalette::Window, Qt::yellow );
+    setPalette( pal );
+    QHBoxLayout * lay = new QHBoxLayout( this );
+    lay->setMargin( 4 );
+    m_label = new QLabel( this );
+    lay->addWidget( m_label );
+    m_label->setWordWrap( true );
+    connect( m_label, SIGNAL( linkActivated( const QString& ) ), this, SIGNAL( action() ) );
+    QToolButton * closeButton = new QToolButton( this );
+    lay->addWidget( closeButton );
+    closeButton->setAutoRaise( true );
+    closeButton->setIcon( closeButton->style()->standardIcon( QStyle::SP_TitleBarCloseButton ) );
+    closeButton->setIconSize( QSize( 32, 32 ) );
+    closeButton->setToolTip( i18n( "Close this message" ) );
+    connect( closeButton, SIGNAL( clicked() ), this, SLOT( hide() ) );
+    hide();
+}
+
+void PageViewTopMessage::display( const QString & message )
+{
+    m_label->setText( message );
+    resize( minimumSizeHint() );
+    show();
+}
+
+
 /*********************/
 /** PageViewToolBar  */
 /*********************/
 
-class ToolBarButton : public QPushButton
+class ToolBarButton : public QToolButton
 {
     public:
         static const int iconSize = 32;
         static const int buttonSize = 40;
 
-        ToolBarButton( QWidget * parent, const ToolBarItem & item, const QPixmap & parentPix );
+        ToolBarButton( QWidget * parent, const ToolBarItem & item );
         int buttonID() const { return m_id; }
 
-    protected:
-        void mouseMoveEvent( QMouseEvent * e );
-        void paintEvent( QPaintEvent * e );
-
     private:
         int m_id;
-        bool m_hovering;
-        const QPixmap & m_background;
 };
 
-ToolBarButton::ToolBarButton( QWidget * parent, const ToolBarItem & item, const QPixmap & pix )
-    : QPushButton( parent ), m_id( item.id ), m_hovering( false ), m_background( pix )
+ToolBarButton::ToolBarButton( QWidget * parent, const ToolBarItem & item )
+    : QToolButton( parent ), m_id( item.id )
 {
-    setMouseTracking( true );
     setCheckable( true );
+    setAutoRaise( true );
     resize( buttonSize, buttonSize );
     setIconSize( QSize( iconSize, iconSize ) );
     setIcon( DesktopIconSet( item.pixmap, iconSize ) );
-    setAttribute( Qt::WA_OpaquePaintEvent );
     // set shortcut if defined
     if ( !item.shortcut.isEmpty() )
         setShortcut( QKeySequence( item.shortcut ) );
@@ -265,40 +305,6 @@ ToolBarButton::ToolBarButton( QWidget * parent, const ToolBarItem & item, const
         setToolTip( item.text );
 }
 
-void ToolBarButton::mouseMoveEvent( QMouseEvent * e )
-{
-    // if hovering changes, update gfx
-    bool hover = QRect( 0, 0, width(), height() ) .contains( e->pos() );
-    if ( m_hovering != hover )
-    {
-        m_hovering = hover;
-        update();
-    }
-}
-
-void ToolBarButton::paintEvent( QPaintEvent * e )
-{
-    // if the button is pressed or we're hovering it, use QPushButton style
-    if ( isChecked() || m_hovering )
-    {
-        QPushButton::paintEvent( e );
-        return;
-    }
-
-    // draw button's pixmap over the parent's background (fake transparency)
-    QPainter p( this );
-    QRect backRect = e->rect();
-    backRect.translate( x(), y() );
-    p.drawPixmap( e->rect().topLeft(), m_background, backRect );
-    QStyleOptionButton opt;
-    opt.initFrom( this );
-    opt.features = QStyleOptionButton::Flat;
-    opt.icon = icon();
-    opt.iconSize = QSize( iconSize, iconSize );
-    opt.text = text();
-    style()->drawControl( QStyle::CE_PushButton, &opt, &p, this );
-}
-
 /* PageViewToolBar */
 
 static const int toolBarGridSize = 40;
@@ -325,6 +331,7 @@ PageViewToolBar::PageViewToolBar( QWidget * parent, QWidget * anchorWidget )
     : QWidget( parent ), d( new ToolBarPrivate )
 {
     setAttribute( Qt::WA_OpaquePaintEvent, true );
+    setMouseTracking( true );
 
     // initialize values of the private data storage structure
     d->anchorWidget = anchorWidget;
@@ -364,7 +371,7 @@ void PageViewToolBar::showItems( Side side, const QLinkedList<ToolBarItem> & ite
     QLinkedList<ToolBarItem>::const_iterator it = items.begin(), end = items.end();
     for ( ; it != end; ++it )
     {
-        ToolBarButton * button = new ToolBarButton( this, *it, d->backgroundPixmap );
+        ToolBarButton * button = new ToolBarButton( this, *it );
         connect( button, SIGNAL( clicked() ), this, SLOT( slotButtonClicked() ) );
         d->buttons.append( button );
     }
@@ -409,7 +416,8 @@ bool PageViewToolBar::eventFilter( QObject * obj, QEvent * e )
 void PageViewToolBar::paintEvent( QPaintEvent * e )
 {
     // paint the internal pixmap over the widget
-    bitBlt( this, e->rect().topLeft(), &d->backgroundPixmap, e->rect() );
+    QPainter p( this );
+    p.drawPixmap( e->rect().topLeft(), d->backgroundPixmap, e->rect() );
 }
 
 void PageViewToolBar::mousePressEvent( QMouseEvent * e )
@@ -421,7 +429,7 @@ void PageViewToolBar::mousePressEvent( QMouseEvent * e )
 
 void PageViewToolBar::mouseMoveEvent( QMouseEvent * e )
 {
-    if ( e->state() != Qt::LeftButton )
+    if ( ( QApplication::mouseButtons() & Qt::LeftButton ) != Qt::LeftButton )
         return;
 
     // compute the nearest side to attach the widget to
