@@ -24,6 +24,7 @@
 #include <qtoolbar.h>
 #include <kaction.h>
 #include <kactioncollection.h>
+#include <kinstance.h>
 #include <klocale.h>
 #include <kiconloader.h>
 #include <kmessagebox.h>
@@ -35,6 +36,7 @@
 
 // local includes
 #include "presentationwidget.h"
+#include "annotationtools.h"
 #include "pagepainter.h"
 #include "core/generator.h"
 #include "core/page.h"
@@ -50,20 +52,24 @@
 // transition effect to the next frame
 struct PresentationFrame
 {
-    const KPDFPage * page;
+    const Okular::Page * page;
     QRect geometry;
 };
 
 
-PresentationWidget::PresentationWidget( QWidget * parent, KPDFDocument * doc )
+PresentationWidget::PresentationWidget( QWidget * parent, Okular::Document * doc )
     : QDialog( parent, Qt::FramelessWindowHint ),
-    m_pressedLink( 0 ), m_handCursor( false ), m_document( doc ), m_frameIndex( -1 ),
-    m_topBar( 0 ), m_pagesEdit( 0 )
+    m_pressedLink( 0 ), m_handCursor( false ), m_drawingEngine( 0 ), m_document( doc ),
+    m_frameIndex( -1 ), m_topBar( 0 ), m_pagesEdit( 0 )
 {
     setModal( true );
     setAttribute( Qt::WA_DeleteOnClose );
     setAttribute( Qt::WA_OpaquePaintEvent );
     setObjectName( "presentationWidget" );
+    QString caption = doc->getMetaData( "DocumentTitle" ).toString();
+    if ( caption.trimmed().isEmpty() )
+        caption = doc->currentDocument().fileName();
+    setWindowTitle( KInstance::makeStdCaption( caption ) );
 
     m_width = -1;
 
@@ -72,6 +78,7 @@ PresentationWidget::PresentationWidget( QWidget * parent, KPDFDocument * doc )
 
     // misc stuff
     setMouseTracking( true );
+    setContextMenuPolicy( Qt::PreventContextMenu );
     m_transitionTimer = new QTimer( this );
     m_transitionTimer->setSingleShot( true );
     connect( m_transitionTimer, SIGNAL( timeout() ), this, SLOT( slotTransitionStep() ) );
@@ -80,12 +87,12 @@ PresentationWidget::PresentationWidget( QWidget * parent, KPDFDocument * doc )
     connect( m_overlayHideTimer, SIGNAL( timeout() ), this, SLOT( slotHideOverlay() ) );
 
     // handle cursor appearance as specified in configuration
-    if ( KpdfSettings::slidesCursor() == KpdfSettings::EnumSlidesCursor::HiddenDelay )
+    if ( Okular::Settings::slidesCursor() == Okular::Settings::EnumSlidesCursor::HiddenDelay )
     {
         KCursor::setAutoHideCursor( this, true );
         KCursor::setHideCursorDelay( 3000 );
     }
-    else if ( KpdfSettings::slidesCursor() == KpdfSettings::EnumSlidesCursor::Hidden )
+    else if ( Okular::Settings::slidesCursor() == Okular::Settings::EnumSlidesCursor::Hidden )
     {
         setCursor( KCursor::blankCursor() );
     }
@@ -98,9 +105,8 @@ PresentationWidget::~PresentationWidget()
     // remove this widget from document observer
     m_document->removeObserver( this );
 
-    // set a new viewport in document if page number differs
-    if ( m_frameIndex != -1 && m_frameIndex != m_document->viewport().pageNumber )
-        m_document->setViewportPage( m_frameIndex/*, PRESENTATION_ID*/ );
+    m_document->removePageAnnotations( m_document->viewport().pageNumber, m_currentPageDrawings );
+    delete m_drawingEngine;
 
     // delete frames
     QVector< PresentationFrame * >::iterator fIt = m_frames.begin(), fEnd = m_frames.end();
@@ -109,7 +115,7 @@ PresentationWidget::~PresentationWidget()
 }
 
 
-void PresentationWidget::notifySetup( const QVector< KPDFPage * > & pageSet, bool /*documentChanged*/ )
+void PresentationWidget::notifySetup( const QVector< Okular::Page * > & pageSet, bool /*documentChanged*/ )
 {
     // delete previous frames (if any (shouldn't be))
     QVector< PresentationFrame * >::iterator fIt = m_frames.begin(), fEnd = m_frames.end();
@@ -120,7 +126,7 @@ void PresentationWidget::notifySetup( const QVector< KPDFPage * > & pageSet, boo
     m_frames.clear();
 
     // create the new frames
-    QVector< KPDFPage * >::const_iterator setIt = pageSet.begin(), setEnd = pageSet.end();
+    QVector< Okular::Page * >::const_iterator setIt = pageSet.begin(), setEnd = pageSet.end();
     float screenRatio = (float)m_height / (float)m_width;
     for ( ; setIt != setEnd; ++setIt )
     {
@@ -143,7 +149,7 @@ void PresentationWidget::notifySetup( const QVector< KPDFPage * > & pageSet, boo
 
     // get metadata from the document
     m_metaStrings.clear();
-    const DocumentInfo * info = m_document->documentInfo();
+    const Okular::DocumentInfo * info = m_document->documentInfo();
     if ( info )
     {
         if ( !info->get( "title" ).isNull() )
@@ -158,22 +164,22 @@ void PresentationWidget::notifySetup( const QVector< KPDFPage * > & pageSet, boo
 void PresentationWidget::notifyViewportChanged( bool /*smoothMove*/ )
 {
     // discard notifications if displaying the summary
-    if ( m_frameIndex == -1 && KpdfSettings::slidesShowSummary() )
+    if ( m_frameIndex == -1 && Okular::Settings::slidesShowSummary() )
         return;
 
     // display the current page
     changePage( m_document->viewport().pageNumber );
 
     // auto advance to the next page if set
-    if ( KpdfSettings::slidesAdvance() )
-        QTimer::singleShot( KpdfSettings::slidesAdvanceTime() * 1000, this, SLOT( slotNextPage() ) );
+    if ( Okular::Settings::slidesAdvance() )
+        QTimer::singleShot( Okular::Settings::slidesAdvanceTime() * 1000, this, SLOT( slotNextPage() ) );
 }
 
 void PresentationWidget::notifyPageChanged( int pageNumber, int changedFlags )
 {
     // check if it's the last requested pixmap. if so update the widget.
-    if ( (changedFlags & DocumentObserver::Pixmap) && pageNumber == m_frameIndex )
-        generatePage();
+    if ( (changedFlags & ( DocumentObserver::Pixmap | DocumentObserver::Annotations ) ) && pageNumber == m_frameIndex )
+        generatePage( changedFlags & DocumentObserver::Annotations );
 }
 
 bool PresentationWidget::canUnloadPixmap( int pageNumber )
@@ -195,13 +201,14 @@ bool PresentationWidget::event( QEvent * e )
     {
         QHelpEvent * he = (QHelpEvent*)e;
 
-        const KPDFLink * link = getLink( he->x(), he->y(), 0 );
+        QRect r;
+        const Okular::Link * link = getLink( he->x(), he->y(), &r );
 
         if ( link )
         {
             QString tip = link->linkTip();
             if ( !tip.isEmpty() )
-                QToolTip::showText( he->globalPos(), tip, this );
+                QToolTip::showText( he->globalPos(), tip, this, r );
         }
         e->accept();
         return true;
@@ -254,6 +261,17 @@ void PresentationWidget::wheelEvent( QWheelEvent * e )
 
 void PresentationWidget::mousePressEvent( QMouseEvent * e )
 {
+    if ( m_drawingEngine )
+    {
+        QRect r = routeMouseDrawingEvent( e );
+        if ( r.isValid() )
+        {
+            m_drawingRect |= r.translated( m_frames[ m_frameIndex ]->geometry.topLeft() );
+            update( m_drawingRect );
+        }
+        return;
+    }
+
     // pressing left button
     if ( e->button() == Qt::LeftButton )
     {
@@ -278,10 +296,29 @@ void PresentationWidget::mousePressEvent( QMouseEvent * e )
 
 void PresentationWidget::mouseReleaseEvent( QMouseEvent * e )
 {
+    if ( m_drawingEngine )
+    {
+        QRect r = routeMouseDrawingEvent( e );
+        (void)r;
+        if ( m_drawingEngine->creationCompleted() )
+        {
+            QList< Okular::Annotation * > annots = m_drawingEngine->end();
+            // manually disable and re-enable the pencil mode, so we can do
+            // cleaning of the actual drawer and create a new one just after
+            // that - that gives continuous drawing
+            togglePencilMode( false );
+            togglePencilMode( true );
+            foreach( Okular::Annotation * ann, annots )
+                m_document->addPageAnnotation( m_frameIndex, ann );
+            m_currentPageDrawings << annots;
+        }
+        return;
+    }
+
     // if releasing on the same link we pressed over, execute it
     if ( m_pressedLink && e->button() == Qt::LeftButton )
     {
-        const KPDFLink * link = getLink( e->x(), e->y() );
+        const Okular::Link * link = getLink( e->x(), e->y() );
         if ( link == m_pressedLink )
             m_document->processLink( link );
         m_pressedLink = 0;
@@ -295,7 +332,7 @@ void PresentationWidget::mouseMoveEvent( QMouseEvent * e )
         return;
 
     // update cursor and tooltip if hovering a link
-    if ( KpdfSettings::slidesCursor() != KpdfSettings::EnumSlidesCursor::Hidden )
+    if ( !m_drawingEngine && Okular::Settings::slidesCursor() != Okular::Settings::EnumSlidesCursor::Hidden )
         testCursorOnLink( e->x(), e->y() );
 
     if ( !m_topBar->isHidden() )
@@ -309,12 +346,23 @@ void PresentationWidget::mouseMoveEvent( QMouseEvent * e )
     }
     else
     {
-        // show the bar if reaching top 2 pixels
-        if ( e->y() <= (geometry().top() + 1) )
-            m_topBar->show();
-        // handle "dragging the wheel" if clicking on its geometry
-        else if ( ( QApplication::mouseButtons() & Qt::LeftButton ) && m_overlayGeometry.contains( e->pos() ) )
-            overlayClick( e->pos() );
+        if ( m_drawingEngine && e->buttons() != Qt::NoButton )
+        {
+            QRect r = routeMouseDrawingEvent( e );
+            {
+                m_drawingRect |= r.translated( m_frames[ m_frameIndex ]->geometry.topLeft() );
+                update( m_drawingRect );
+            }
+        }
+        else
+        {
+            // show the bar if reaching top 2 pixels
+            if ( e->y() <= (geometry().top() + 1) )
+                m_topBar->show();
+            // handle "dragging the wheel" if clicking on its geometry
+            else if ( ( QApplication::mouseButtons() & Qt::LeftButton ) && m_overlayGeometry.contains( e->pos() ) )
+                overlayClick( e->pos() );
+        }
     }
 }
 
@@ -330,7 +378,7 @@ void PresentationWidget::paintEvent( QPaintEvent * pe )
         m_topBar = new QToolBar( this );
         m_topBar->setObjectName( "presentationBar" );
         m_topBar->setIconSize( QSize( 32, 32 ) );
-        m_topBar->addAction( KIcon("1leftarrow"), i18n("Previous Page"), this, SLOT( slotPrevPage() ) );
+        m_topBar->addAction( KIcon( layoutDirection() == Qt::RightToLeft ? "1rightarrow" : "1leftarrow" ), i18n("Previous Page"), this, SLOT( slotPrevPage() ) );
         m_pagesEdit = new QLineEdit( m_topBar );
         QSizePolicy sp = m_pagesEdit->sizePolicy();
         sp.setHorizontalPolicy( QSizePolicy::Minimum );
@@ -341,7 +389,13 @@ void PresentationWidget::paintEvent( QPaintEvent * pe )
         m_pagesEdit->setValidator( validator );
         m_topBar->addWidget( m_pagesEdit );
         connect( m_pagesEdit, SIGNAL( returnPressed() ), this, SLOT( slotPageChanged() ) );
-        m_topBar->addAction( KIcon("1rightarrow"), i18n("Next Page"), this, SLOT( slotNextPage() ) );
+        m_topBar->addAction( KIcon( layoutDirection() == Qt::RightToLeft ? "1leftarrow" : "1rightarrow" ), i18n("Next Page"), this, SLOT( slotNextPage() ) );
+        m_topBar->addSeparator();
+        QAction * drawingAct = m_topBar->addAction( KIcon( "pencil" ), i18n( "Toggle Drawing Mode" ) );
+        drawingAct->setCheckable( true );
+        connect( drawingAct, SIGNAL( toggled( bool ) ), this, SLOT( togglePencilMode( bool ) ) );
+        QAction * eraseDrawingAct = m_topBar->addAction( KIcon( "eraser" ), i18n( "Erase Drawings" ) );
+        connect( eraseDrawingAct, SIGNAL( triggered() ), this, SLOT( clearDrawings() ) );
         QWidget *spacer = new QWidget(m_topBar);
         spacer->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::MinimumExpanding );
         m_topBar->addWidget( spacer );
@@ -358,7 +412,7 @@ void PresentationWidget::paintEvent( QPaintEvent * pe )
         m_document->addObserver( this );
 
         // show summary if requested
-        if ( KpdfSettings::slidesShowSummary() )
+        if ( Okular::Settings::slidesShowSummary() )
             generatePage();
     }
 
@@ -377,7 +431,7 @@ void PresentationWidget::paintEvent( QPaintEvent * pe )
         if ( !r.isValid() )
             continue;
 #ifdef ENABLE_PROGRESS_OVERLAY
-        if ( KpdfSettings::slidesShowProgress() && r.intersects( m_overlayGeometry ) )
+        if ( Okular::Settings::slidesShowProgress() && r.intersects( m_overlayGeometry ) )
         {
             // backbuffer the overlay operation
             QPixmap backPixmap( r.size() );
@@ -400,12 +454,20 @@ void PresentationWidget::paintEvent( QPaintEvent * pe )
         // copy the rendered pixmap to the screen
         painter.drawPixmap( r.topLeft(), m_lastRenderedPixmap, r );
     }
+    if ( m_drawingEngine && m_drawingRect.intersects( pe->rect() ) )
+    {
+        const QRect & geom = m_frames[ m_frameIndex ]->geometry;
+        painter.save();
+        painter.translate( geom.topLeft() );
+        m_drawingEngine->paint( &painter, geom.width(), geom.height(), m_drawingRect.intersect( pe->rect() ) );
+        painter.restore();
+    }
     painter.end();
 }
 // </widget events>
 
 
-const KPDFLink * PresentationWidget::getLink( int x, int y, QRect * geometry ) const
+const Okular::Link * PresentationWidget::getLink( int x, int y, QRect * geometry ) const
 {
     // no links on invalid pages
     if ( geometry && !geometry->isNull() )
@@ -415,7 +477,7 @@ const KPDFLink * PresentationWidget::getLink( int x, int y, QRect * geometry ) c
 
     // get frame, page and geometry
     const PresentationFrame * frame = m_frames[ m_frameIndex ];
-    const KPDFPage * page = frame->page;
+    const Okular::Page * page = frame->page;
     const QRect & frameGeometry = frame->geometry;
 
     // compute normalized x and y
@@ -427,7 +489,8 @@ const KPDFLink * PresentationWidget::getLink( int x, int y, QRect * geometry ) c
         return 0;
 
     // check if 1) there is an object and 2) it's a link
-    const ObjectRect * object = page->getObjectRect( ObjectRect::Link, nx, ny );
+    QRect d = KGlobalSettings::desktopGeometry( const_cast< PresentationWidget * >( this ) );
+    const Okular::ObjectRect * object = page->getObjectRect( Okular::ObjectRect::Link, nx, ny, d.width(), d.height() );
     if ( !object )
         return 0;
 
@@ -439,12 +502,12 @@ const KPDFLink * PresentationWidget::getLink( int x, int y, QRect * geometry ) c
     }
 
     // return the link pointer
-    return (KPDFLink *)object->pointer();
+    return (Okular::Link *)object->pointer();
 }
 
 void PresentationWidget::testCursorOnLink( int x, int y )
 {
-    const KPDFLink * link = getLink( x, y, 0 );
+    const Okular::Link * link = getLink( x, y, 0 );
 
     // only react on changes (in/out from a link)
     if ( (link && !m_handCursor) || (!link && m_handCursor) )
@@ -488,15 +551,15 @@ void PresentationWidget::changePage( int newPage )
     m_pagesEdit->setText( QString::number( m_frameIndex + 1 ) );
     m_pagesEdit->blockSignals( signalsBlocked );
 
-    // if pixmap not inside the KPDFPage we request it and wait for
+    // if pixmap not inside the Okular::Page we request it and wait for
     // notifyPixmapChanged call or else we can proceed to pixmap generation
     if ( !frame->page->hasPixmap( PRESENTATION_ID, pixW, pixH ) )
     {
         // operation will take long: set busy cursor
         QApplication::setOverrideCursor( KCursor::workingCursor() );
         // request the pixmap
-        QLinkedList< PixmapRequest * > request;
-        request.push_back( new PixmapRequest( PRESENTATION_ID, m_frameIndex, pixW, pixH, rot, PRESENTATION_PRIO ) );
+        QLinkedList< Okular::PixmapRequest * > request;
+        request.push_back( new Okular::PixmapRequest( PRESENTATION_ID, m_frameIndex, pixW, pixH, rot, PRESENTATION_PRIO ) );
         m_document->requestPixmaps( request );
         // restore cursor
         QApplication::restoreOverrideCursor();
@@ -506,9 +569,17 @@ void PresentationWidget::changePage( int newPage )
         // make the background pixmap
         generatePage();
     }
+
+    // set a new viewport in document if page number differs
+    if ( m_frameIndex != -1 && m_frameIndex != m_document->viewport().pageNumber )
+    {
+        // remove the drawing on the old page before switching
+        clearDrawings();
+        m_document->setViewportPage( m_frameIndex, PRESENTATION_ID );
+    }
 }
 
-void PresentationWidget::generatePage()
+void PresentationWidget::generatePage( bool disableTransition )
 {
     if ( m_lastRenderedPixmap.isNull() )
         m_lastRenderedPixmap = QPixmap( m_width, m_height );
@@ -526,22 +597,30 @@ void PresentationWidget::generatePage()
 
     // generate the top-right corner overlay
 #ifdef ENABLE_PROGRESS_OVERLAY
-    if ( KpdfSettings::slidesShowProgress() && m_frameIndex != -1 )
+    if ( Okular::Settings::slidesShowProgress() && m_frameIndex != -1 )
         generateOverlay();
 #endif
 
     // start transition on pages that have one
-    const KPDFPageTransition * transition = m_frameIndex != -1 ?
-        m_frames[ m_frameIndex ]->page->getTransition() : 0;
-    if ( transition )
-        initTransition( transition );
-    else {
-        KPDFPageTransition trans = defaultTransition();
+    if ( !disableTransition )
+    {
+        const Okular::PageTransition * transition = m_frameIndex != -1 ?
+            m_frames[ m_frameIndex ]->page->getTransition() : 0;
+        if ( transition )
+            initTransition( transition );
+        else {
+            Okular::PageTransition trans = defaultTransition();
+            initTransition( &trans );
+        }
+    }
+    else
+    {
+        Okular::PageTransition trans = defaultTransition( Okular::Settings::EnumSlidesTransition::Replace );
         initTransition( &trans );
     }
 
     // update cursor + tooltip
-    if ( KpdfSettings::slidesCursor() != KpdfSettings::EnumSlidesCursor::Hidden )
+    if ( Okular::Settings::slidesCursor() != Okular::Settings::EnumSlidesCursor::Hidden )
     {
         QPoint p = mapFromGlobal( QCursor::pos() );
         testCursorOnLink( p.x(), p.y() );
@@ -611,7 +690,7 @@ void PresentationWidget::generateContentsPage( int pageNum, QPainter & p )
     geom.translate( -geom.left(), -geom.top() );
 
     // draw the page using the shared PagePainter class
-    int flags = PagePainter::Accessibility;
+    int flags = PagePainter::Accessibility | PagePainter::Annotations;
     PagePainter::paintPageOnPainter( &p, frame->page, PRESENTATION_ID, flags,
                                      geom.width(), geom.height(), geom );
 
@@ -624,7 +703,7 @@ void PresentationWidget::generateContentsPage( int pageNum, QPainter & p )
     for ( int i = 0; i < rects.count(); i++ )
     {
         const QRect & r = rects[i];
-        p.fillRect( r, KpdfSettings::slidesBackgroundColor() );
+        p.fillRect( r, Okular::Settings::slidesBackgroundColor() );
     }
 }
 
@@ -742,11 +821,46 @@ void PresentationWidget::generateOverlay()
 }
 
 
+QRect PresentationWidget::routeMouseDrawingEvent( QMouseEvent * e )
+{
+    const QRect & geom = m_frames[ m_frameIndex ]->geometry;
+    const Okular::Page * page = m_frames[ m_frameIndex ]->page;
+
+    AnnotatorEngine::EventType eventType = AnnotatorEngine::Press;
+    if ( e->type() == QEvent::MouseMove )
+        eventType = AnnotatorEngine::Move;
+    else if ( e->type() == QEvent::MouseButtonRelease )
+        eventType = AnnotatorEngine::Release;
+
+    // find out the pressed button
+    AnnotatorEngine::Button button = AnnotatorEngine::None;
+    Qt::MouseButtons buttonState = ( eventType == AnnotatorEngine::Move ) ? e->buttons() : e->button();
+    if ( buttonState == Qt::LeftButton )
+        button = AnnotatorEngine::Left;
+    else if ( buttonState == Qt::RightButton )
+        button = AnnotatorEngine::Right;
+    static bool hasclicked = false;
+    if ( eventType == AnnotatorEngine::Press )
+        hasclicked = true;
+
+    double nX = ( (double)e->x() - (double)geom.left() ) / (double)geom.width();
+    double nY = ( (double)e->y() - (double)geom.top() ) / (double)geom.height();
+    QRect ret;
+    if ( hasclicked && nX >= 0 && nX < 1 && nY >= 0 && nY < 1 )
+        ret = m_drawingEngine->event( eventType, button, nX, nY, geom.width(), geom.height(), page );
+
+    if ( eventType == AnnotatorEngine::Release )
+        hasclicked = false;
+
+    return ret;
+}
+
+
 
 void PresentationWidget::slotNextPage()
 {
     // loop when configured
-    if ( m_frameIndex == (int)m_frames.count() - 1 && KpdfSettings::slidesLoop() )
+    if ( m_frameIndex == (int)m_frames.count() - 1 && Okular::Settings::slidesLoop() )
         m_frameIndex = -1;
 
     if ( m_frameIndex < (int)m_frames.count() - 1 )
@@ -754,13 +868,13 @@ void PresentationWidget::slotNextPage()
         // go to next page
         changePage( m_frameIndex + 1 );
         // auto advance to the next page if set
-        if ( KpdfSettings::slidesAdvance() )
-            QTimer::singleShot( KpdfSettings::slidesAdvanceTime() * 1000, this, SLOT( slotNextPage() ) );
+        if ( Okular::Settings::slidesAdvance() )
+            QTimer::singleShot( Okular::Settings::slidesAdvanceTime() * 1000, this, SLOT( slotNextPage() ) );
     }
     else
     {
 #ifdef ENABLE_PROGRESS_OVERLAY
-        if ( KpdfSettings::slidesShowProgress() )
+        if ( Okular::Settings::slidesShowProgress() )
             generateOverlay();
 #endif
         if ( m_transitionTimer->isActive() )
@@ -781,13 +895,13 @@ void PresentationWidget::slotPrevPage()
         changePage( m_frameIndex - 1 );
 
         // auto advance to the next page if set
-        if ( KpdfSettings::slidesAdvance() )
-            QTimer::singleShot( KpdfSettings::slidesAdvanceTime() * 1000, this, SLOT( slotNextPage() ) );
+        if ( Okular::Settings::slidesAdvance() )
+            QTimer::singleShot( Okular::Settings::slidesAdvanceTime() * 1000, this, SLOT( slotNextPage() ) );
     }
     else
     {
 #ifdef ENABLE_PROGRESS_OVERLAY
-        if ( KpdfSettings::slidesShowProgress() )
+        if ( Okular::Settings::slidesShowProgress() )
             generateOverlay();
 #endif
         if ( m_transitionTimer->isActive() )
@@ -849,155 +963,187 @@ void PresentationWidget::slotPageChanged()
     changePage( p - 1 );
 }
 
-const KPDFPageTransition PresentationWidget::defaultTransition() const
+void PresentationWidget::togglePencilMode( bool on )
 {
-    return defaultTransition( KpdfSettings::slidesTransition() );
+    if ( on )
+    {
+        QString colorstring = Okular::Settings::slidesPencilColor().name();
+        // FIXME this should not be recreated every time
+        QDomDocument doc( "engine" );
+        QDomElement root = doc.createElement( "engine" );
+        root.setAttribute( "color", colorstring );
+        doc.appendChild( root );
+        QDomElement annElem = doc.createElement( "annotation" );
+        root.appendChild( annElem );
+        annElem.setAttribute( "type", "Ink" );
+        annElem.setAttribute( "color", colorstring );
+        annElem.setAttribute( "width", "2" );
+        m_drawingEngine = new SmoothPathEngine( root );
+    }
+    else
+    {
+        delete m_drawingEngine;
+        m_drawingEngine = 0;
+        m_drawingRect = QRect();
+    }
 }
 
-const KPDFPageTransition PresentationWidget::defaultTransition( int type ) const
+void PresentationWidget::clearDrawings()
+{
+    m_document->removePageAnnotations( m_document->viewport().pageNumber, m_currentPageDrawings );
+    m_currentPageDrawings.clear();
+}
+
+
+const Okular::PageTransition PresentationWidget::defaultTransition() const
+{
+    return defaultTransition( Okular::Settings::slidesTransition() );
+}
+
+const Okular::PageTransition PresentationWidget::defaultTransition( int type ) const
 {
     switch ( type )
     {
-        case KpdfSettings::EnumSlidesTransition::BlindsHorizontal:
+        case Okular::Settings::EnumSlidesTransition::BlindsHorizontal:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Blinds );
-            transition.setAlignment( KPDFPageTransition::Horizontal );
+            Okular::PageTransition transition( Okular::PageTransition::Blinds );
+            transition.setAlignment( Okular::PageTransition::Horizontal );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::BlindsVertical:
+        case Okular::Settings::EnumSlidesTransition::BlindsVertical:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Blinds );
-            transition.setAlignment( KPDFPageTransition::Vertical );
+            Okular::PageTransition transition( Okular::PageTransition::Blinds );
+            transition.setAlignment( Okular::PageTransition::Vertical );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::BoxIn:
+        case Okular::Settings::EnumSlidesTransition::BoxIn:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Box );
-            transition.setDirection( KPDFPageTransition::Inward );
+            Okular::PageTransition transition( Okular::PageTransition::Box );
+            transition.setDirection( Okular::PageTransition::Inward );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::BoxOut:
+        case Okular::Settings::EnumSlidesTransition::BoxOut:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Box );
-            transition.setDirection( KPDFPageTransition::Outward );
+            Okular::PageTransition transition( Okular::PageTransition::Box );
+            transition.setDirection( Okular::PageTransition::Outward );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::Dissolve:
+        case Okular::Settings::EnumSlidesTransition::Dissolve:
         {
-            return KPDFPageTransition( KPDFPageTransition::Dissolve );
+            return Okular::PageTransition( Okular::PageTransition::Dissolve );
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::GlitterDown:
+        case Okular::Settings::EnumSlidesTransition::GlitterDown:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Glitter );
+            Okular::PageTransition transition( Okular::PageTransition::Glitter );
             transition.setAngle( 270 );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::GlitterRight:
+        case Okular::Settings::EnumSlidesTransition::GlitterRight:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Glitter );
+            Okular::PageTransition transition( Okular::PageTransition::Glitter );
             transition.setAngle( 0 );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::GlitterRightDown:
+        case Okular::Settings::EnumSlidesTransition::GlitterRightDown:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Glitter );
+            Okular::PageTransition transition( Okular::PageTransition::Glitter );
             transition.setAngle( 315 );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::Random:
+        case Okular::Settings::EnumSlidesTransition::Random:
         {
             return defaultTransition( KRandom::random() % 18 );
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::SplitHorizontalIn:
+        case Okular::Settings::EnumSlidesTransition::SplitHorizontalIn:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Split );
-            transition.setAlignment( KPDFPageTransition::Horizontal );
-            transition.setDirection( KPDFPageTransition::Inward );
+            Okular::PageTransition transition( Okular::PageTransition::Split );
+            transition.setAlignment( Okular::PageTransition::Horizontal );
+            transition.setDirection( Okular::PageTransition::Inward );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::SplitHorizontalOut:
+        case Okular::Settings::EnumSlidesTransition::SplitHorizontalOut:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Split );
-            transition.setAlignment( KPDFPageTransition::Horizontal );
-            transition.setDirection( KPDFPageTransition::Outward );
+            Okular::PageTransition transition( Okular::PageTransition::Split );
+            transition.setAlignment( Okular::PageTransition::Horizontal );
+            transition.setDirection( Okular::PageTransition::Outward );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::SplitVerticalIn:
+        case Okular::Settings::EnumSlidesTransition::SplitVerticalIn:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Split );
-            transition.setAlignment( KPDFPageTransition::Vertical );
-            transition.setDirection( KPDFPageTransition::Inward );
+            Okular::PageTransition transition( Okular::PageTransition::Split );
+            transition.setAlignment( Okular::PageTransition::Vertical );
+            transition.setDirection( Okular::PageTransition::Inward );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::SplitVerticalOut:
+        case Okular::Settings::EnumSlidesTransition::SplitVerticalOut:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Split );
-            transition.setAlignment( KPDFPageTransition::Vertical );
-            transition.setDirection( KPDFPageTransition::Outward );
+            Okular::PageTransition transition( Okular::PageTransition::Split );
+            transition.setAlignment( Okular::PageTransition::Vertical );
+            transition.setDirection( Okular::PageTransition::Outward );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::WipeDown:
+        case Okular::Settings::EnumSlidesTransition::WipeDown:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Wipe );
+            Okular::PageTransition transition( Okular::PageTransition::Wipe );
             transition.setAngle( 270 );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::WipeRight:
+        case Okular::Settings::EnumSlidesTransition::WipeRight:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Wipe );
+            Okular::PageTransition transition( Okular::PageTransition::Wipe );
             transition.setAngle( 0 );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::WipeLeft:
+        case Okular::Settings::EnumSlidesTransition::WipeLeft:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Wipe );
+            Okular::PageTransition transition( Okular::PageTransition::Wipe );
             transition.setAngle( 180 );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::WipeUp:
+        case Okular::Settings::EnumSlidesTransition::WipeUp:
         {
-            KPDFPageTransition transition( KPDFPageTransition::Wipe );
+            Okular::PageTransition transition( Okular::PageTransition::Wipe );
             transition.setAngle( 90 );
             return transition;
             break;
         }
-        case KpdfSettings::EnumSlidesTransition::Replace:
+        case Okular::Settings::EnumSlidesTransition::Replace:
         default:
-            return KPDFPageTransition( KPDFPageTransition::Replace );
+            return Okular::PageTransition( Okular::PageTransition::Replace );
             break;
     }
     // should not happen, just make gcc happy
-    return KPDFPageTransition();
+    return Okular::PageTransition();
 }
 
 /** ONLY the TRANSITIONS GENERATION function from here on **/
-void PresentationWidget::initTransition( const KPDFPageTransition *transition )
+void PresentationWidget::initTransition( const Okular::PageTransition *transition )
 {
     // if it's just a 'replace' transition, repaint the screen
-    if ( transition->type() == KPDFPageTransition::Replace )
+    if ( transition->type() == Okular::PageTransition::Replace )
     {
         update();
         return;
     }
 
-    const bool isInward = transition->direction() == KPDFPageTransition::Inward;
-    const bool isHorizontal = transition->alignment() == KPDFPageTransition::Horizontal;
+    const bool isInward = transition->direction() == Okular::PageTransition::Inward;
+    const bool isHorizontal = transition->alignment() == Okular::PageTransition::Horizontal;
     const float totalTime = transition->duration();
 
     m_transitionRects.clear();
@@ -1005,7 +1151,7 @@ void PresentationWidget::initTransition( const KPDFPageTransition *transition )
     switch( transition->type() )
     {
             // split: horizontal / vertical and inward / outward
-        case KPDFPageTransition::Split:
+        case Okular::PageTransition::Split:
         {
             const int steps = isHorizontal ? 100 : 75;
             if ( isHorizontal )
@@ -1063,7 +1209,7 @@ void PresentationWidget::initTransition( const KPDFPageTransition *transition )
         } break;
 
             // blinds: horizontal(l-to-r) / vertical(t-to-b)
-        case KPDFPageTransition::Blinds:
+        case Okular::PageTransition::Blinds:
         {
             const int blinds = isHorizontal ? 8 : 6;
             const int steps = m_width / (4 * blinds);
@@ -1104,7 +1250,7 @@ void PresentationWidget::initTransition( const KPDFPageTransition *transition )
         } break;
 
             // box: inward / outward
-        case KPDFPageTransition::Box:
+        case Okular::PageTransition::Box:
         {
             const int steps = m_width / 10;
             if ( isInward )
@@ -1148,7 +1294,7 @@ void PresentationWidget::initTransition( const KPDFPageTransition *transition )
         } break;
 
             // wipe: implemented for 4 canonical angles
-        case KPDFPageTransition::Wipe:
+        case Okular::PageTransition::Wipe:
         {
             const int angle = transition->angle();
             const int steps = (angle == 0) || (angle == 180) ? m_width / 8 : m_height / 8;
@@ -1202,7 +1348,7 @@ void PresentationWidget::initTransition( const KPDFPageTransition *transition )
         } break;
 
             // dissolve: replace 'random' rects
-        case KPDFPageTransition::Dissolve:
+        case Okular::PageTransition::Dissolve:
         {
             const int gridXsteps = 50;
             const int gridYsteps = 38;
@@ -1241,7 +1387,7 @@ void PresentationWidget::initTransition( const KPDFPageTransition *transition )
         } break;
 
             // glitter: similar to dissolve but has a direction
-        case KPDFPageTransition::Glitter:
+        case Okular::PageTransition::Glitter:
         {
             const int gridXsteps = 50;
             const int gridYsteps = 38;
@@ -1333,15 +1479,15 @@ void PresentationWidget::initTransition( const KPDFPageTransition *transition )
         } break;
 
         // implement missing transitions (a binary raster engine needed here)
-        case KPDFPageTransition::Fly:
+        case Okular::PageTransition::Fly:
 
-        case KPDFPageTransition::Push:
+        case Okular::PageTransition::Push:
 
-        case KPDFPageTransition::Cover:
+        case Okular::PageTransition::Cover:
 
-        case KPDFPageTransition::Uncover:
+        case Okular::PageTransition::Uncover:
 
-        case KPDFPageTransition::Fade:
+        case Okular::PageTransition::Fade:
 
         default:
             update();
